/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package com.voztele.sipspy;

import java.io.*;
import java.net.InetAddress;
import java.net.UnknownHostException;

public class SipMessage implements Serializable {

   public final static int SIP_REQUEST = 1;

   public final static int SIP_RESPONSE = 2;

   private String messageFrom = null;

   private String messageTo = null;

   private String messageFromPort;

   private String messageToPort;

   private String messageTime = null;

   private String messageString = null;

   private String messageFirstLine = null;

   private String messageFirstViaHost = null;

   private String messageFirstViaPort = null;

   private String messageStatusInfo = null;

   private String messageTransactionId = null;

   private String debugLine = null;

   private String callId = null;

   int type;

   String beforeDebug;

   String afterDebug;

   public SipMessage() {
   }

   public SipMessage(int type, String messageFrom, String messageTo, String messageFromPort,
         String messageToPort, String messageTime, String messageFirstLine,
         String messageFirstViaHost, String messageFirstViaPort, String messageString,
         String messageStatusInfo, String messageTransactionId, String debugLine, String callId) {
      this.messageFrom = messageFrom;
      this.messageTo = messageTo;
      this.messageFromPort = messageFromPort;
      this.messageToPort = messageToPort;
      this.messageTime = messageTime;
      this.messageString = messageString;
      this.messageFirstLine = messageFirstLine;
      this.messageFirstViaHost = messageFirstViaHost;
      this.messageFirstViaPort = messageFirstViaPort;
      this.messageStatusInfo = messageStatusInfo;
      this.messageTransactionId = messageTransactionId;
      this.debugLine = debugLine;
      this.callId = callId;
      this.type = type;
   }

   public void setFrom(String from) {
      messageFrom = from;
   }

   public void setTo(String to) {
      messageTo = to;
   }

   public void setTime(String time) {
      messageTime = time;
   }

   public void setMessageString(String str) {
      messageString = str;
   }

   public void setFirstLine(String FirstLine) {
      messageFirstLine = FirstLine;
   }

   public void setStatusInfo(String statusInfo) {
      messageStatusInfo = statusInfo;
   }

   public void setTransactionId(String transactionId) {
      messageTransactionId = transactionId;
   }

   public String getFrom() {
      return messageFrom;
   }

   public String getTo() {
      return messageTo;
   }

   public String getToPort() {
      return messageToPort;
   }

   public String getFromPort() {
      return messageFromPort;
   }

   public String getTime() {
      return messageTime;
   }

   public String getMessageString() {
      return messageString;
   }

   public String getFirstLine() {
      return messageFirstLine;
   }

   public String getStatusInfo() {
      return messageStatusInfo;
   }

   public String getTransactionId() {
      return messageTransactionId;
   }

   public String getCallId() {
      return callId;
   }

   public void setCallId(String callId) {
      this.callId = callId;
   }

   public String getFirstViaHost() {
      InetAddress ia;
      try {
         ia = InetAddress.getByName(messageFirstViaHost);
      } catch (UnknownHostException e) {
         return messageFirstViaHost;
      }
      return ia.getHostAddress();
   }

   public String getFirstViaPort() {
      return messageFirstViaPort;
   }

   public boolean isRequest() {
      return type == SIP_REQUEST;
   }
}
