/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package com.voztele.sipspy;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.TextArea;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.log4j.Logger;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import com.voztele.sipspy.connections.Connection;
import com.voztele.sipspy.connections.ConnectionManager;
import com.voztele.sipspy.connections.ConnectionsFrame;
import com.voztele.sipspy.connections.SessionFileParser;
import com.voztele.sipspy.flowpanel.StaticTracesCanvas;
import com.voztele.sipspy.ui.PercentLayout;
import com.voztele.sipspy.ui.PercentLayoutConstraint;

public class SipSpy extends JApplet implements ActionListener{
    
    private File configFile;
    private Logger log = Logger.getLogger(this.getClass());
    private ArrayList switchable_opts = null;
    private ArrayList nonapplet_opts  = null;
    // All the components inside the second panel
    protected JLabel sessionsLabel;
    protected SpySessionsList spySessionsList;
    protected JButton messageContentButton;
    protected TextArea messageContentTextArea;
    protected ConnectionsFrame connectionsFrame;
    protected static String stackId;
    private boolean isApplet = true;
    private int windowWidth=600;
    private int windowHeight=400;
    private String title = "VozTele.com SipSpy";
    public StaticTracesCanvas tracesCanvas;
    private JPanel panel;
    
    /**
     * A JApplet is itself a top-level Swing-Hierarchy FRAME, but when
     * running standalone, the FRAME must be instantiated. Anyway, we will
     * keep a reference to the frame to abstract ourselves from being
     * Applet or standalone.
     */
    private JFrame frame;
    private JPanel rightBigPanel;
    private JPanel rightSuperPanel;
    private JPanel leftBigPanel;
    private JPanel firstLeftSubPanel;
    private JPanel secondLeftSubPanel;
    private JPanel thirdLeftSubPanel;
    private ConnectionManager connections ;
    private Document configDoc;
    /** applet constructor.*/
    public SipSpy(){
        this(true);
    }

    /**When executing as an applet, the constructor called will be the no-arg one,
     * from there, this constructor is called with true as argument.
     */
    public SipSpy(boolean amIapplet) {
        super();
        setApplet(amIapplet);
        /* positioning and diminsioning the frame/applet */
        int w = windowWidth + 10;
        int h = windowHeight + 10;
        if (isApplet){
            setPanel((JPanel)this.getContentPane());
            setSize( w, h );
        } else {
            /* we are running as a stand alone program */
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            this.frame = new JFrame();
            frame.addWindowListener(new WindowAdapter(){
                public void windowClosing(WindowEvent e){
                    TransformerFactory tf = TransformerFactory.newInstance();
                    try{
                        tf.setAttribute("indent-number", new Integer(4));                       
                    }catch (IllegalArgumentException e4){
                    }
                    Transformer t=null;
                    try {
                        t = tf.newTransformer();
                    } catch (TransformerConfigurationException e3) {
                        e3.printStackTrace();
                    } 
                    t.setOutputProperty(OutputKeys.INDENT, "yes");
                    t.setOutputProperty(OutputKeys.METHOD, "xml");
                    t.setOutputProperty(OutputKeys.MEDIA_TYPE, "text/xml");
                    FileOutputStream fos=null;

                    try {
                        if(configFile.exists())
                            configFile.delete();
                        configFile.createNewFile();
                        fos = new FileOutputStream(configFile);
                    } catch (FileNotFoundException e2) {
                        e2.printStackTrace();
                    } catch (IOException e4) {
                        JOptionPane.showMessageDialog(frame, e4.getMessage(), "File Error",
                                JOptionPane.ERROR_MESSAGE);
                        System.exit(0);
                    }
                    try {
                        t.transform(new DOMSource(configDoc), new StreamResult(new OutputStreamWriter(fos)));
                    } catch (TransformerException e1) {
                        e1.printStackTrace();
                    }
                    System.exit(0);
                }
            });
            frame.setLocation(0,0);
            frame.setSize(screenSize.width,screenSize.height);
            frame.setTitle( title );
            setPanel((JPanel)frame.getContentPane());
        }

    }

    public void setPanel(JPanel panel) {
        this.panel = panel;
    }
    
    public JPanel getPanel(){
        return this.panel;
    }

    private void setupLayout(){

        /********************** The main container ****************************/
        
        Container container = this.getPanel();
        container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
        container.setBackground(Color.black);
        
        /** Right super-panel*/

        /**   Panel gordo de flechitas y dibujitos ********************************/
        
        rightBigPanel = new JPanel();
        // If put to False: we see the container's background
        rightBigPanel.setOpaque(false);
        rightBigPanel.setLayout(new PercentLayout());
        container.add(rightBigPanel);

        // Sub right panel:
        // topx %, topy %, width %, height % 73, 100-> 65, 95
        PercentLayoutConstraint firstPanelConstraint =
            new PercentLayoutConstraint(30, 5, 70, 95);
        if(spySessionsList!=null)
            spySessionsList.removeAll();
        spySessionsList = new SpySessionsList(connections);
        
        spySessionsList.addListSelectionListener(new ListSelectionListener(){
            public void valueChanged(ListSelectionEvent arg0) {
                spySessionsList.updateTracesCanvas(arg0);
            }
        });
        spySessionsList.setForeground(Color.black);
        spySessionsList.setFont(new Font("Dialog", 1, 14));
        
        // width, height
        this.setSize(670, 620);
        this.setLocation(0, 0);

        /* makes the panel visible */
        getPanel().setBackground(new Color((int)128,(int)128,(int)255));
        getPanel().setVisible( true );
        getPanel().repaint();       

        // Warning: to put before for the canvas!!!!
        JTextArea messageContentTextArea = new JTextArea();
        messageContentButton = new JButton("SIP Message:");
        StaticTracesCanvas tracesCanvas;
        tracesCanvas=new StaticTracesCanvas(messageContentTextArea,"unknown",this);
        setTracesCanvas(tracesCanvas);
        spySessionsList.setTracesCanvas(tracesCanvas);
        tracesCanvas.setOpaque(true);

        // The ScrollPane for the Canvas
        JScrollPane scroll = new JScrollPane();
        scroll.getViewport().add(tracesCanvas);
        tracesCanvas.setScrollPane(scroll);
        rightBigPanel.add(scroll, firstPanelConstraint);
        // Sub left panel:
        firstLeftSubPanel = new JPanel();
        firstLeftSubPanel.setBackground(Color.black);
        // Top, left, bottom, right
        //firstLeftSubPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 7, 5));
        PercentLayoutConstraint myPanelConstraint =new PercentLayoutConstraint(30, 0, 70, 5);
        rightBigPanel.add(firstLeftSubPanel, myPanelConstraint);
        firstLeftSubPanel.add(createToolBar());

        /******** Panel de la izquierda, con opciones y men???s********************************/

        // left panel: 
        leftBigPanel = new JPanel();
        leftBigPanel.setBackground(Color.black);
        // rows, columns
        // secondPanel.setLayout(new GridLayout(3,1,0,0) );
        leftBigPanel.setLayout(new BorderLayout());
        // topx %, topy %, width %, height %
        PercentLayoutConstraint secondPanelConstraint =
            new PercentLayoutConstraint(0, 0, 30, 100);
        rightBigPanel.add(leftBigPanel, secondPanelConstraint);
        
        /****************** SECOND SUB PANEL ****************************************/
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(2, 1, 0, 0));
        leftBigPanel.add(panel, BorderLayout.CENTER);
        secondLeftSubPanel = new JPanel();
        secondLeftSubPanel.setBackground(Color.black);
        secondLeftSubPanel.setLayout(new BorderLayout());
        panel.add(secondLeftSubPanel);

        sessionsLabel = new JLabel("Available Sessions:");
        sessionsLabel.setToolTipText("All the sessions currently available");
        // Alignment of the text
        sessionsLabel.setHorizontalAlignment(AbstractButton.CENTER);
        // Color of the text
        sessionsLabel.setForeground(Color.black);
        // Size of the text
        sessionsLabel.setFont(new Font("Dialog", 1, 14));
        // If put to true: we see the label's background
        sessionsLabel.setOpaque(true);
        sessionsLabel.setBackground(Color.lightGray);
        sessionsLabel.setBorder(BorderFactory.createLineBorder(Color.darkGray));
        secondLeftSubPanel.add(sessionsLabel, BorderLayout.NORTH);

        JScrollPane scrollList = new JScrollPane(spySessionsList);
        spySessionsList.setVisible(true);
        secondLeftSubPanel.add(scrollList, BorderLayout.CENTER);

        /******************** THIRD Left SUB PANEL ****************************************/

        thirdLeftSubPanel = new JPanel();
        thirdLeftSubPanel.setBackground(Color.black);
        thirdLeftSubPanel.setLayout(new BorderLayout());
        //secondPanel.add(thirdSubPanel);
        panel.add(thirdLeftSubPanel);

        messageContentButton.setToolTipText("Display all the content of the current SIP message");
        // Alignment of the text
        messageContentButton.setHorizontalAlignment(AbstractButton.CENTER);
        // Color of the text
        messageContentButton.setForeground(Color.black);
        // Size of the text
        messageContentButton.setFont(new Font("Dialog", 1, 14));
        // If put to true: we see the label's background
        messageContentButton.setOpaque(true);
        messageContentButton.setBackground(Color.lightGray);
        messageContentButton.setBorder(BorderFactory.createLineBorder(Color.darkGray));
        messageContentTextArea.setBackground(Color.white);
        messageContentTextArea.setEditable(false);
        messageContentTextArea.setFont(new Font("Dialog", 1, 12));
        messageContentTextArea.setForeground(Color.black);
        thirdLeftSubPanel.add(messageContentButton, BorderLayout.NORTH);
        JScrollPane msgScroll = new JScrollPane(messageContentTextArea);
        thirdLeftSubPanel.add(msgScroll, BorderLayout.CENTER);
        validateTree();
    }
    
    private JToolBar createToolBar() {
        JToolBar tb = new JToolBar(JToolBar.HORIZONTAL);
        tb.setFloatable(true);
        tb.setBorderPainted(true);
        tb.add(makeButton("connect.png", "Connect", "Open Connections Pane", "Connect"));
        tb.add(makeButton("page_white.png","Clean","Clear the sesion list","Clear"));
        tb.add(makeButton("disk.png","Save","Save selected Session","Save"));
        tb.add(makeButton("disk_multiple.png","Save All","Save all the sessions","Save All"));
        tb.add(makeButton("folder.png","Open","Open a sessions file","Open"));
        tb.add(makeButton("picture_save.png","Capture","Capture JPG","Capture"));
        tb.add(makeButton("cross.png","Remove","Remove Sessions from list","Remove"));
        tb.add(makeButton("database_refresh.png","RemoteSave","Save selected Session","Remote Save"));
        tb.add(makeButton("flag_green.png","Replay","Replay Remote Session","Replay Remote"));
        tb.add(makeButton("wesip_logo_small.gif","About","About...","About"));
        return tb;
    }
    
    protected JButton makeButton(String imageName,String actionCommand,String toolTipText,String altText) {
        JButton button = new JButton();
        button.setActionCommand(actionCommand);
        button.setToolTipText(toolTipText);
        button.addActionListener(this);
        button.setIcon(Utils.getIcon(imageName));
        button.setText(altText);
        
        return button;
    }

    public void setLeftBigPanel(JPanel secondPanel) {
        this.leftBigPanel = secondPanel;
    }
    
    public JPanel getLeftBigPanel(){
        return this.leftBigPanel;
    }

    public void setTracesCanvas(StaticTracesCanvas tracesCanvas2) {
        this.tracesCanvas = tracesCanvas2;
        
    }
    
    public StaticTracesCanvas getTracesCanvas(){
        return this.tracesCanvas;
    }

    // This method is only used by the GUI proxy!!!!!
    // We do not need to use the prompt!!!
    public static void main(String args[]) {
        SipSpy sipSpy = new SipSpy(false);
        sipSpy.init();
        sipSpy.start();
    }
    
    public void setApplet(boolean b) {
        this.isApplet = b;
    }
    
    public boolean isApplet(){
        return this.isApplet;
    }

    public void init(){
    }

    private JMenuBar buildBarMenu() {
        switchable_opts = new ArrayList();
        nonapplet_opts  = new ArrayList();

        /**Options for the File Menu.
         * Connect: Connect to agents.
         * Open: Open Traces File.
         * Close: Close Traces File.
         * Save: Save Traces File.
         * Save as...: Save Traces File.
         * Exit: call(Exit: call(Exit: call(Exit:... calls an infinite loop.
         */
        JMenuItem new_menuitem = new JMenuItem("Connect");
            new_menuitem.addActionListener( this );
        JMenuItem open_menuitem = new JMenuItem("Open");
            open_menuitem.addActionListener( this );
        JMenuItem close_menuitem = new JMenuItem("Close");
            close_menuitem.addActionListener( this );
        JMenuItem save_menuitem = new JMenuItem("Save");
            save_menuitem.addActionListener( this );
        JMenuItem saveas_menuitem = new JMenuItem("Save as...");
            saveas_menuitem.addActionListener( this );
        JMenuItem exit_menuitem = new JMenuItem("Exit");
            exit_menuitem.addActionListener( this );
        JMenuItem help_menuitem = new JMenuItem("Help");
            help_menuitem.addActionListener( this );
        JMenuItem about_menuitem = new JMenuItem("About SipSpy");
            about_menuitem.addActionListener( this );
        JMenuItem displaySessions_menuitem = new JMenuItem("Display Sessions");
            displaySessions_menuitem.addActionListener( this );
        JMenuItem refresh_menuitem = new JMenuItem("Refresh");
            
        switchable_opts.add(  close_menuitem );
        switchable_opts.add(   save_menuitem );
        switchable_opts.add( saveas_menuitem );
        switchable_opts.add(help_menuitem);
        switchable_opts.add(displaySessions_menuitem);
        
        nonapplet_opts.add(  close_menuitem );
        nonapplet_opts.add(   save_menuitem );
        nonapplet_opts.add( saveas_menuitem );
        nonapplet_opts.add(   open_menuitem );
        nonapplet_opts.add(   exit_menuitem );
        
        JMenu file_menu = new JMenu("File");
        JMenu help_menu = new JMenu("Help");
        JMenu sessions_menu = new JMenu("Sessions");

        file_menu.add(    new_menuitem );
        file_menu.add(   open_menuitem );
        file_menu.add(  close_menuitem );
        file_menu.addSeparator();
        file_menu.add(   save_menuitem );
        file_menu.add( saveas_menuitem );
        file_menu.addSeparator();
        file_menu.add(   exit_menuitem );
        
        help_menu.add(   help_menuitem );
        help_menu.add(  about_menuitem );
    
        sessions_menu.add(displaySessions_menuitem);
        sessions_menu.add(refresh_menuitem);
        
        JMenuBar menu_bar = new JMenuBar();
        menu_bar.add(file_menu);
        menu_bar.add(help_menu);
        menu_bar.add(sessions_menu);
        menu_bar.add(about_menuitem);

        /* if we run as applet, desable the non-applet options from menus */
        if ( isApplet)
            for( int i=0 ; i<nonapplet_opts.size() ; i++ )
                ((JMenuItem)nonapplet_opts.get(i)).setEnabled(false);
        return menu_bar;

    }

    public void actionPerformed(ActionEvent e) {
        String cmd = e.getActionCommand();
        try {
            if (cmd.equals("Open"))
            {
                JFileChooser chooser = new JFileChooser( "." );
                if (chooser.showOpenDialog(this.frame)!=JFileChooser.APPROVE_OPTION )
                    return;
                String fileName = chooser.getSelectedFile().getPath();
                spySessionsList.removeAll();
                File f = new File(fileName);
                FileInputStream fis = new FileInputStream(f);
                if(!f.exists() || !f.canRead())return;
                SessionFileParser sfp = new SessionFileParser();
                sfp.addSessionsListener(spySessionsList);
                sfp.parse(fis);
                fis.close();
                return;
            }
            if (cmd.equals("Capture"))
            {
                JFileChooser chooser = new JFileChooser(".");
                chooser.setFileFilter(new FileFilter(){

                    public boolean accept(File f) {
                        if(f.getName().endsWith(".gif")) {
                            return true;
                        }
                        return false;
                    }

                    public String getDescription() {
                        return "Gif picture";
                    }
                    
                });
                if (chooser.showSaveDialog(this.frame)!=JFileChooser.APPROVE_OPTION )
                    return;
                if(!chooser.getSelectedFile().exists())chooser.getSelectedFile().createNewFile();
                if(!chooser.getSelectedFile().canWrite())return;
                Dimension d = tracesCanvas.getPreferredSize();
                BufferedImage bufferedImage = new BufferedImage(d.width,d.height,BufferedImage.TYPE_INT_RGB);
                Graphics2D g = bufferedImage.createGraphics();
                tracesCanvas.paintAll(g);
                log.debug(bufferedImage.getHeight() +":"+ bufferedImage.getWidth());
                log.debug("result:"+ImageIO.write(bufferedImage, "png", chooser.getSelectedFile()));        
                return;
            }
            if (cmd.equals("Remove"))
            {
                spySessionsList.removeSelected(); 
                return;
            }
            if (cmd.equals("About"))
            {
            	showAbout();
                return;
            }
            if (cmd.equals("Clean"))
            {
                spySessionsList.removeAll();
                return;
            }
            /* Close the current file - DONE */
            if (cmd.equals("Close"))
            {
                setupLayout();
                return;
            }
            /* Saves the current script into file - Done */
            if (cmd.equals("Save"))
            {
                JFileChooser chooser = new JFileChooser( "." );
                if (chooser.showSaveDialog(frame)!=JFileChooser.APPROVE_OPTION)
                    return;
                String file_name = chooser.getSelectedFile().getPath();
                File f = new File(file_name);
                SpySession ss;
                f.createNewFile();
                FileOutputStream fos=null;
                try {
                    fos = new FileOutputStream(f);
                } catch (FileNotFoundException e2) {
                    e2.printStackTrace();
                }
                fos.write("<?xml version='1.0' encoding='ISO-8859-1'?>\n<messages>\n".getBytes());
                Object[] spySessionVector = spySessionsList.getSelectedValues();
                for(int i=0;i<spySessionVector.length;i++){
                    ss=(SpySession)spySessionVector[i];
                    Iterator ite = ss.iterator();
                    SipMessage sm = null;
                    while(ite.hasNext()){
                        sm = (SipMessage) ite.next();
                        fos.write(("<message\n"+
                                "type=\""+(sm.isRequest()?"1":"2")+"\"\n"+
                                "from=\""+sm.getFrom()+"\"\n"+
                                "to=\""+sm.getTo()+"\"\n"+
                                "fromPort=\""+sm.getFromPort()+"\"\n"+
                                "toPort=\""+sm.getToPort()+"\"\n"+
                                "time=\""+sm.getTime()+"\"\n"+
                                "isSender=\"false\"\n"+
                                "transactionId=\""+sm.getTransactionId()+"\"\n"+
                                "callId=\""+sm.getCallId()+"\"\n"+
                                "firstLine=\""+sm.getFirstLine()+"\"\n"+
                                "firstViaHost=\""+sm.getFirstViaHost()+"\"\n"+
                                "firstViaPort=\""+sm.getFirstViaPort()+"\"\n"+
                                "debugLine=\"0\"\n>\n" +
                                "<![CDATA["+sm.getMessageString()+
                                "]]>\n</message>\n\n").getBytes());
                    }
                }
                fos.write("</messages>".getBytes());
                fos.close();
                return;
            }
            /* Saves the current script into file - Done */
            if (cmd.equals("RemoteSave"))
            {
                String name = JOptionPane.showInputDialog(this, "File name");
                SpySession ss;
                ByteArrayOutputStream baos = new ByteArrayOutputStream(3000);
                //baos.write("<?xml version='1.0' encoding='ISO-8859-1'?>\n<messages>\n".getBytes());
                Object[] spySessionVector = spySessionsList.getSelectedValues();
                for(int i=0;i<spySessionVector.length;i++){
                    ss=(SpySession)spySessionVector[i];
                    Iterator ite = ss.iterator();
                    SipMessage sm = null;
                    while(ite.hasNext()){
                        sm = (SipMessage) ite.next();
                        baos.write(("<message\n"+
                                "type=\""+(sm.isRequest()?"1":"2")+"\"\n"+
                                "from=\""+sm.getFrom()+"\"\n"+
                                "to=\""+sm.getTo()+"\"\n"+
                                "fromPort=\""+sm.getFromPort()+"\"\n"+
                                "toPort=\""+sm.getToPort()+"\"\n"+
                                "time=\""+sm.getTime()+"\"\n"+
                                "isSender=\"false\"\n"+
                                "transactionId=\""+sm.getTransactionId()+"\"\n"+
                                "callId=\""+sm.getCallId()+"\"\n"+
                                "firstLine=\""+sm.getFirstLine()+"\"\n"+
                                "firstViaHost=\""+sm.getFirstViaHost()+"\"\n"+
                                "firstViaPort=\""+sm.getFirstViaPort()+"\"\n"+
                                "debugLine=\"0\"\n>\n" +
                                "<![CDATA["+sm.getMessageString()+
                                "]]>\n</message>\n\n").getBytes());
                    }
                }
               // baos.write("</messages>".getBytes());
                baos.close();
                LinkedList spyagents = new LinkedList();
                for(int i=0;i<connections.getConnectionCount();i++){
                    Connection c = connections.getConnection(i);
                    if(c.getConnected().booleanValue()){
                        spyagents.add(c);
                    }
                }
                if(spyagents.size()==0){
                    JOptionPane.showMessageDialog(this, "No connected spyAgents !!", "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                Object[] spyAgentarray=spyagents.toArray();
                Connection selected = (Connection)JOptionPane.showInputDialog(this,"Select a spyAgent:","SpyAgent selection",
                        JOptionPane.PLAIN_MESSAGE,null,spyAgentarray,spyAgentarray[0]);
                if(selected==null)
                	return;
                selected.sendFile(name,baos.toByteArray());
                return;
            }
            if (cmd.equals("Replay"))
            {
                LinkedList spyagents = new LinkedList();
                for(int i=0;i<connections.getConnectionCount();i++){
                    Connection c = connections.getConnection(i);
                    if(c.getConnected().booleanValue()){
                        spyagents.add(c);
                    }
                }
                if(spyagents.size()==0){
                    JOptionPane.showMessageDialog(this, "No connected spyAgents !!", "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                Object[] spyAgentarray=spyagents.toArray();
                Connection selected = (Connection)JOptionPane.showInputDialog(this,"Select a spyAgent:","SpyAgent selection",
                        JOptionPane.PLAIN_MESSAGE,null,spyAgentarray,spyAgentarray[0]);
                if(selected==null)
                	return;
                List names = selected.listSessions();
                Object[] nameArray=names.toArray();
                String namewanted = (String)JOptionPane.showInputDialog(this,"Select a session:","session selection",
                        JOptionPane.PLAIN_MESSAGE,null,nameArray,nameArray[0]);
                selected.sendCommand("replay", namewanted);
                return;
            }

            /* Saves the tree under a different name - Done */
            if (e.getActionCommand().equals("Save All"))
            {
                JFileChooser chooser = new JFileChooser( "." );
                if (chooser.showSaveDialog(frame)!=JFileChooser.APPROVE_OPTION)
                    return;
                String file_name = chooser.getSelectedFile().getPath();
                File f = new File(file_name);
                f.createNewFile();
                
                FileOutputStream fos=null;
                try {
                    fos = new FileOutputStream(f);
                } catch (FileNotFoundException e2) {
                    e2.printStackTrace();
                }
                fos.write("<?xml version='1.0' encoding='ISO-8859-1'?>\n<messages>\n".getBytes());

                for(int i=0;i<spySessionsList.getSessionCount();i++){
                    SpySession ss = spySessionsList.getSession(i);
                    Iterator ite = ss.iterator();
                    SipMessage sm = null;
                    while(ite.hasNext()){
                        sm = (SipMessage) ite.next();
                        fos.write(("<message\n"+
                                "type=\""+(sm.isRequest()?"1":"2")+"\"\n"+
                                "from=\""+sm.getFrom()+"\"\n"+
                                "to=\""+sm.getTo()+"\"\n"+
                                "fromPort=\""+sm.getFromPort()+"\"\n"+
                                "toPort=\""+sm.getToPort()+"\"\n"+
                                "time=\""+sm.getTime()+"\"\n"+
                                "isSender=\"false\"\n"+
                                "transactionId=\""+sm.getTransactionId()+"\"\n"+
                                "callId=\""+sm.getCallId()+"\"\n"+
                                "firstLine=\""+sm.getFirstLine()+"\"\n"+
                                "firstViaHost=\""+sm.getFirstViaHost()+"\"\n"+
                                "firstViaPort=\""+sm.getFirstViaPort()+"\"\n"+
                                "debugLine=\"0\"\n>\n<![CDATA["+sm.getMessageString()+
                                "]]>\n</message>\n\n").getBytes());
                    }
                }
                fos.write("</messages>".getBytes());
                fos.close();
                return;
            }

            if (e.getActionCommand().equals("Connect"))
            {
                if(connectionsFrame == null)
                    connectionsFrame = new ConnectionsFrame(connections);
                connectionsFrame.setVisible(true);
                return;
            }
            /* Exits the application - Done */
            if (cmd.equals("Exit"))
            {
                System.exit(0);
            }
        }catch(Exception ex) {
            ex.printStackTrace();
        }
    }

    private void showAbout() {
		AboutFrame.show(this);
	}

	private void updateLayout(ArrayList spySessions2) {
        this.spySessionsList.removeAll();
        for(int i=0;i<spySessions2.size();i++){
            this.spySessionsList.sessionEvent((SpySession) spySessions2.get(i));            
        }
    }

    /**
     * @return Returns the stackId.
     */
    public static String getStackId() {
        return stackId;
    }

    /**
     * @param stackId The stackId to set.
     */
    public static void setStackId(String stackId) {
        SipSpy.stackId = stackId;
    }

    public Document getConfigDoc() {
        return this.configDoc;
    }
    
    public void start(){
    	configure();
        try {
            this.connections = new ConnectionManager(this);
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        }
        setupLayout();
        if (isApplet){
            this.setVisible(true);
            String sessionUrl =getParameter("defaultSession");
            InputStream sessionInputStream = null;
            try{
            if(sessionUrl!=null){
            	HttpURLConnection connection = (HttpURLConnection) new URL(sessionUrl).openConnection();
            	sessionInputStream = connection.getInputStream();
            }
            spySessionsList.removeAll();
            SessionFileParser sfp = new SessionFileParser();
            sfp.addSessionsListener(spySessionsList);
            sfp.parse(sessionInputStream);
            sessionInputStream.close();
            }catch(Exception e){
            	if(sessionInputStream!=null)
					try {
						sessionInputStream.close();
					} catch (IOException e1) {
					}
            }
        }else{
            frame.setVisible(true);
        }
    }
    
    public void configure(){
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        InputStream configInputStream = null;
        DocumentBuilder builder;
        try {
            builder = factory.newDocumentBuilder();
            if (!isApplet) {
                configFile = new File("conf/SipSpy.xml");
                if (configFile.exists()) {
                    configInputStream = new FileInputStream(configFile);
                }
            } else {
                String confURL = getParameter("configURL");
                
                if(confURL!=null){
                    HttpURLConnection connection = (HttpURLConnection) new URL(confURL).openConnection();
                    configInputStream= connection.getInputStream();
                }
            }
            if(configInputStream!=null){
                configDoc = builder.parse(configInputStream);
                if (!configDoc.getDocumentElement().getNodeName().equals("SipSpy")) {
                    log.error("SipSpy Configuration File does not have a SipSpy root tag");
                    configDoc.removeChild(configDoc.getDocumentElement());
                    configDoc.appendChild(configDoc.createElement("SipSpy"));
                }
            }else{
                DOMImplementation impl = builder.getDOMImplementation();
                this.configDoc = impl.createDocument(null, "SipSpy", null);
            }
        } catch (IOException e) {
            log.error(e.getMessage());
        } catch (ParserConfigurationException e) {
            log.error(e.getMessage());
            e.printStackTrace();
            System.exit(0);
        } catch (SAXException e) {
            log.error(e.getMessage());
        }

    }
}