/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


package com.voztele.sipspy;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.TreeSet;

import org.apache.log4j.Logger;

/**
*This class stores a sorted list messages for logging. 
*
*/

public class SpySession extends TreeSet implements MessageProvider {

	private Logger log = Logger.getLogger(this.getClass());
	protected String description;
	private String callId;
	private ArrayList messageListeners = new ArrayList();
	protected static long startTime;

	static {
		startTime = -1;
	}
	
	public SpySession(){
		super();
	}
	
	/** Constructor given callId and a comparator.
	*@param callId is the call id for which to store the log.
	*@param comp is the comparator to sort the log records.
	*/

	public SpySession(String callId, Comparator comp) {
		super(comp);
		this.callId = callId;
	}
	
	/** set a descriptive string for this log (for id purposes).
	 *@param description is the decriptive string to add.
	 */
	public void addDescription(String description) {
		this.description = description;
	}

	/** Add a comparable object to the messgageLog 
	*@param obj is the comparable object to add to the message log.
	*/
	public synchronized boolean add(Object obj) {
		SipMessage msg = (SipMessage) obj;
		
		log.debug("Adding SipMsg: " + msg.getFirstLine() + "\nto "+callId
				+" received at "+msg.getTime());
		long ts = Long.parseLong(msg.getTime());
		boolean ret;
		if (ts < startTime || startTime < 0)
			startTime = ts;
		ret = super.add(obj);
		for(int i=0;i<messageListeners.size();i++){
			((MessageListener)messageListeners.get(i)).messageEvent((SipMessage)obj);
		}
		return ret;
	}

	/**
	 * @return Returns the callId.
	 */
	public String getCallId() {
		return callId;
	}

	/**
	 * @param callId The callId to set.
	 */
	public void setCallId(String callId) {
		this.callId = callId;
	}

	public void addMessageListener(MessageListener ml) {
		this.messageListeners.add(ml);
	}

	public void removeMessageListener(MessageListener ml) {
		this.messageListeners.remove(ml);
	}

	public ArrayList getMessageListeners() {
		return messageListeners;
	}
	
	public String getFirstLine(){
		return ((SipMessage)first()).getFirstLine();
		
	}

}
