/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package com.voztele.sipspy;

import java.awt.Color;
import java.awt.Component;
import java.util.Enumeration;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JList;
import javax.swing.event.ListSelectionEvent;

import org.apache.log4j.Logger;

import com.voztele.sipspy.flowpanel.StaticTracesCanvas;

public class SpySessionsList extends JList implements SpySessionListener{

	private static ImageIcon images[] = new ImageIcon[7];
	private Logger log = Logger.getLogger(this.getClass());
	private DefaultListModel sessions = new DefaultListModel();
	protected int index = 0;
	private StaticTracesCanvas tracesCanvas = null;

	public SpySessionsList(SpySessionProvider ssp) {
		setModel(sessions);
		setCellRenderer(new MyListCellRenderer());
		ssp.addSessionsListener(this);
	}
	
	public void messageEvent(SpySession ss, SipMessage tm, boolean lateMessage) {
		
	}

	public void sessionEvent(SpySession spySession) {
		log.debug("Session Added:" + spySession.getCallId());
		if(sessions.contains(spySession)){
			log.debug("Session already in the list");
			return;
		}
		sessions.addElement(spySession);
		repaint();
	}

	public void updateTracesCanvas(ListSelectionEvent e) {
		if (sessions == null || sessions.isEmpty())
			return;
		log.debug("Selected Values:"+getSelectedValues());
		tracesCanvas.refreshTracesCanvas(getSelectedValues());
	}


	/**
	 * @return Returns the tracesCanvas.
	 */
	public StaticTracesCanvas getTracesCanvas() {
		return tracesCanvas;
	}

	public SpySession getSession(int i){
		return (SpySession) sessions.get(i);
	}

	/**
	 * @param tracesCanvas The tracesCanvas to set.
	 */
	public void setTracesCanvas(StaticTracesCanvas tracesCanvas) {
		this.tracesCanvas = tracesCanvas;
	}

	public void removeAll(){
		sessions.removeAllElements();
	}
	
	public int getSessionCount(){
		return sessions.size();
	}
	
	private void loadImages(){
	    images[0] = Utils.getIcon("bullet_red.gif");
	    images[1] = Utils.getIcon("bullet_blue.gif");
	    images[2] = Utils.getIcon("bullet_orange.gif");
	    images[3] = Utils.getIcon("bullet_green.gif");
	    images[4] = Utils.getIcon("bullet_purple.gif");
	    images[5] = Utils.getIcon("bullet_black.gif");
	    images[6] = Utils.getIcon("bullet_yellow.gif");
	}
	
	class MyListCellRenderer extends DefaultListCellRenderer {
		Logger log = Logger.getLogger(this.getClass());
		
		public Component getListCellRendererComponent(JList list,Object value,int index,
				boolean isSelected,boolean cellHasFocus){
        	SpySession ss = (SpySession) value;
        	String first = ss.getFirstLine();
        	Color c = Color.white;
        	ImageIcon ii=null;
        	Component retValue = super.getListCellRendererComponent(list, 
        			first, index, isSelected, cellHasFocus);
			if(first.startsWith("INVITE"))ii = Utils.getIcon("bullet_red.gif");
			else if(first.startsWith("REGISTER"))ii=Utils.getIcon("bullet_blue.gif");
			else if(first.startsWith("OPTIONS"))ii=Utils.getIcon("bullet_yellow.gif");
			else if(first.startsWith("SUBSCRIBE"))ii=Utils.getIcon("bullet_green.gif");
			else if(first.startsWith("BYE"))ii=Utils.getIcon("bullety_black.gif");
			else if(first.startsWith("MESSAGE"))ii=Utils.getIcon("bullet_orange.gif");
			else if(first.startsWith("SIP"))ii=Utils.getIcon("bullet_star.gif");
			if(isSelected)c=c.darker();
			retValue.setBackground(c);
			setIcon(ii);
			return retValue;
        }

     }

	public void removeSelected() {
		Object i[] = getSelectedValues();
		for(int j=0;j<i.length;j++){
			sessions.removeElement(i[j]);
		}
	}

}
