/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package com.voztele.sipspy.connections;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.swing.JTable;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.voztele.sipspy.LogComparator;
import com.voztele.sipspy.MessageListener;
import com.voztele.sipspy.SipMessage;
import com.voztele.sipspy.SipSpy;
import com.voztele.sipspy.SpySession;
import com.voztele.sipspy.SpySessionListener;
import com.voztele.sipspy.SpySessionProvider;

public class ConnectionManager implements SpySessionProvider, MessageListener {
	
	private Logger log = Logger.getLogger(this.getClass());

	public static final int ADDED = 0;
	public static final int REMOVED = 1;
	private Hashtable sessions = new Hashtable();
	private ArrayList connections = new ArrayList();
	private Element element;
	private SipSpy sipSpy;
	private ArrayList listeners = new ArrayList();
	private ArrayList spySessionListeners = new ArrayList();
	private ConnectionsFrame connectionsFrame;
	
	public ConnectionManager(SipSpy sp) throws ParserConfigurationException{
		this.sipSpy=sp;
		Document doc = sp.getConfigDoc();
		NodeList nl = null;
		nl = doc.getDocumentElement().getElementsByTagName("connections");
		if(nl!=null && nl.getLength()>0){
			element= (Element) nl.item(0);
			NodeList nl2 = element.getElementsByTagName("connection");
			for(int i=0;i<nl2.getLength();i++){
				Connection c = new Connection(this,(Element)nl2.item(i));
				connections.add(c);
			}
		}else{
			element = doc.createElement("connections");
			doc.getDocumentElement().appendChild(element);
		}
	}

	public int numConnections() {
		return this.connections.size();
	}

	public Connection getConnection(int i) {
		return (Connection) this.connections.get(i);
	}

	public void addConnection(Connection c) {
		connections.add(c);
		for(int i=0;i<listeners.size();i++){
			((ConnectionsListener)listeners.get(i))
			.connectionsEvent(new ConnectionsEvent(this,ADDED));
		}
	}
	
	public void removeConnection(Connection c){
		try {
			c.setConnected(new Boolean(false));
		} catch (NumberFormatException e) {
			e.printStackTrace();
		} catch (UnknownHostException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		connections.remove(c);
		this.element.removeChild(c.getConfigElm());
		for(int i=0;i<listeners.size();i++){
			((ConnectionsListener)listeners.get(i))
			.connectionsEvent(new ConnectionsEvent(this,REMOVED));
		}
	}

	public Element getConfigElem() {
		return this.element;
	}

	public void addListener(ConnectionsTableModel model) {
		this.listeners .add(model);
	}
	
	public void removeListener(ConnectionsTableModel model){
		this.listeners.remove(model);
	}

	public JTable getTable() {
		return this.connectionsFrame.getTable();
	}
	
	public void setConnectionsFrame(ConnectionsFrame jf) {
		this.connectionsFrame = jf;
	}

	public void addSessionsListener(SpySessionListener ml) {
		this.spySessionListeners.add(ml);
	}

	public void removeSessionsListener(SpySessionListener ml) {
		spySessionListeners.remove(ml);
	}

	public ArrayList getSessionsListeners() {
		return spySessionListeners;
	}

	public void messageEvent(SipMessage sipMsg) {
		log.debug("Message event:" + sipMsg.getFirstLine());
		String callId = sipMsg.getCallId();
		SpySession mySession=(SpySession) sessions.get(callId);
		if (mySession == null){
			mySession = new SpySession(callId,new LogComparator());
			sessions.put(callId, mySession);
			mySession.add(sipMsg);
			fireSessionEvent(mySession);
		}else{
			mySession.add(sipMsg);
		}
		
	}

	private void fireSessionEvent(SpySession ss) {
		for(int i=0;i<spySessionListeners.size();i++){
			((SpySessionListener)spySessionListeners.get(i)).sessionEvent(ss);
		}
	}
	
	public void finishDocument() {
		
	}

	/**
	 * @param document The document to set.
	 */
	public void setConfigElement(Element elem) {
		this.element = elem;
	}
	
	public Document getConfigDoc(){
		return sipSpy.getConfigDoc();
	}

	/**
	 * @return Returns the connectionsFrame.
	 */
	public ConnectionsFrame getConnectionsFrame() {
		return connectionsFrame;
	}
	
	public int getConnectionCount(){
		return connections.size();
	}

}
