/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package com.voztele.sipspy.connections;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;

import com.voztele.sipspy.Utils;

public class ConnectionsFrame extends JFrame implements ActionListener {

	Logger log = Logger.getLogger(this.getClass());

	private JTable table;

	private ConnectionManager connections;

	private ConnectionsTableModel tableModel;

	private Connection c;

	private JScrollPane scrollPane;

	private LittleCellEditor lce;

	public ConnectionsFrame(ConnectionManager conns) {
		super("ConnectionManager Manager");
		this.connections = conns;
		conns.setConnectionsFrame(this);

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				setVisible(false);
				dispose();
			}
		});

		JPanel contentPane = new JPanel();
		contentPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);

		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		contentPane.setLayout(gridbag);

		c.gridwidth = GridBagConstraints.REMAINDER;
		c.anchor = GridBagConstraints.NORTHWEST;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1.0;

		JToolBar toolBar = createToolBar();
		gridbag.setConstraints(toolBar, c);
		contentPane.add(toolBar);

		JLabel label = new JLabel("ConnectionManager:");
		c.insets = new Insets(5, 0, 5, 0);
		gridbag.setConstraints(label, c);
		contentPane.add(label);

		c.anchor = GridBagConstraints.NORTH;
		c.insets = new Insets(5, 0, 10, 0);
		c.weighty = 1.0;
		c.gridheight = GridBagConstraints.REMAINDER;

		tableModel = new ConnectionsTableModel(connections);
		table = new JTable(tableModel);
		lce = new LittleCellEditor();
		table.getColumnModel().getColumn(ConnectionsTableModel.DEVICE)
				.setCellRenderer(lce);
		table.getColumnModel().getColumn(ConnectionsTableModel.DEVICE)
				.setCellEditor(lce);
		table.getColumnModel().getColumn(2).setPreferredWidth(50);
		table.getColumnModel().getColumn(3).setPreferredWidth(25);
		table.getColumnModel().getColumn(7).setPreferredWidth(10);
		table.getColumnModel().getColumn(5).setPreferredWidth(25);
		table.setRowSelectionAllowed(true);
		table.setColumnSelectionAllowed(false);

		TableColumn column = table.getColumnModel().getColumn(0);
		column.setMinWidth(20);
		column.setMaxWidth(20);
		column.setPreferredWidth(20);

		table.getTableHeader().setFont(new Font("Dialog", Font.BOLD, 10));
		table.setFont(new Font("Dialog", Font.BOLD, 14));

		scrollPane = new JScrollPane(table);
		c.fill = GridBagConstraints.BOTH;
		gridbag.setConstraints(scrollPane, c);
		contentPane.add(scrollPane);

		setLocation(420, 200);

		pack();
		setVisible(true);
	}

	private JToolBar createToolBar() {
		JToolBar tb = new JToolBar();
		tb.setFloatable(false);
		tb.setBorderPainted(true);
		tb.add(makeButton("add.png", "Add", "Add a Connection", "Add"));
		tb.add(makeButton("cross.png", "Remove",
				"Remove the connection", "Remove"));
		tb.add(makeButton("flag_green.png",
				"StartAll", "Start all ConnectionManager", "StartAll"));
		tb.add(makeButton("flag_red.png",
				"StopAll", "Stop All ConnectionManager", "StopAll"));
		return tb;
	}

	private void error(String msg, String title) {
		JOptionPane.showMessageDialog(ConnectionsFrame.this, msg, title,
				JOptionPane.ERROR_MESSAGE);
	}

	public JScrollPane getScrollPane() {
		return scrollPane;
	}

	public TableModel getTableModel() {
		return tableModel;
	}

	protected JButton makeButton(String imageName, String actionCommand,
			String toolTipText, String altText) {
		JButton button = new JButton();
		button.setActionCommand(actionCommand);
		button.setToolTipText(toolTipText);
		button.addActionListener(this);
		button.setIcon(Utils.getIcon(imageName));
		button.setText(altText);

		return button;
	}

	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equals("Add")) {
			c = new Connection(connections);
			try {
				new NewConnectionFrame(c, this);
			} catch (ParserConfigurationException e1) {
				e1.printStackTrace();
			}
			scrollPane.doLayout();
			scrollPane.setSize(getWidth(), 50 + tableModel.getRowCount() * 25);
			scrollPane.validate();
			doLayout();
			validateTree();
		} else if (arg0.getActionCommand().equals("StopAll")) {
		} else if (arg0.getActionCommand().equals("StartAll")) {
		} else if (arg0.getActionCommand().equals("Remove")) {
			int con = connections.getTable().getSelectedRow();
			Connection c = ((ConnectionsTableModel) connections.getTable()
					.getModel()).getConnection(con);
			connections.removeConnection(c);
		}
	}

	public Dimension getPreferredSize() {
		int width = 600;
		int height = 225 + scrollPane.getHeight();
		return new Dimension(width, height);
	}

	/**
	 * @return Returns the table.
	 */
	public JTable getTable() {
		return table;
	}
}
