/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


package com.voztele.sipspy.connections;

import java.io.IOException;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import org.apache.log4j.Logger;

import com.voztele.sipspy.Utils;


public class ConnectionsTableModel extends AbstractTableModel implements ConnectionListener,
ConnectionsListener{
	
	private static Logger log =Logger.getLogger(ConnectionsTableModel.class);
	
	public static final int ICON = 0;
	public static final int IP = 1;
	public static final int PORT = 2;
	public static final int RCVDMSGS= 3;
	public static final int REGEXP = 4;
	public static final int DEVICE = 5;
	public static final int FILTER = 6;
	public static final int CONNECTED = 7;
    public static final int USERNAME = 8;
    public static final int PASSWORD = 9;
	
	private static final long serialVersionUID = 1957986056934148598L;
	private ConnectionManager connections;
	private JTable table;
    
    public ConnectionsTableModel(ConnectionManager connections) {
        this.connections = connections;
        this.table = connections.getTable();
        connections.addListener(this);
        for (int i = 0; i < connections.numConnections(); i++) {
            connections.getConnection(i).addListener(this);
        }
    }
    
    void cleanup() {
        for (int i = 0; i < connections.numConnections(); i++) {
            connections.getConnection(i).removeListener(this);
        }
        connections.removeListener(this);
    }

    public void connectionEvent(Connection c,int eventType) {
    	int i = getIndex(c);
        if (i >= 0) {
            fireTableCellUpdated(i, 0);
        }
    }
    
    private int getIndex(Connection connection) {
        for (int i = 0; i < connections.numConnections(); i++) {
            if (connections.getConnection(i) == connection) {
                return i;
            }
        }
        return -1;
    }
    
    public Class getColumnClass(int columnIndex) {
        switch(columnIndex) {
        	case ICON:
        		return ImageIcon.class;
        	case IP:
        	case PORT:
        	case RCVDMSGS:
        	case REGEXP:
        	case DEVICE:
        	case FILTER:
        		return String.class;
        	case CONNECTED:
        		return Boolean.class;
            case USERNAME:
            case PASSWORD:
               return String.class;
    		default:
    			return String.class;
        }
    }
    
    public int getRowCount() {
        return connections.numConnections();
    }
    
    public int getColumnCount() {
        return 10;
    }
    
    public String getColumnName(int column) {
        switch(column) {
	    	case ICON:
	    		return "";
	    	case IP:
	    		return "Internet Address";
	    	case PORT:
	    		return "Port";
	    	case RCVDMSGS:
	    		return "Received Messages";
	    	case REGEXP:
	    		return "RegExp";
	    	case DEVICE:
	    		return "device";
	    	case FILTER:
	    		return "filter";
	    	case CONNECTED:
	    		return "Connected";
            case USERNAME:
               return "Username";
            case PASSWORD:
               return "Password";
			default:
				return "dontknow";
        }
    }
    
    public Object getValueAt(int row, int column) {
        switch(column) {
        	case ICON:
                try {
					return Utils.getIcon(getConnection(row).getState());
				} catch (IOException e) {
					return null;
				}
        	case IP:
        		return getConnection(row).getIp();
        	case PORT:
        		return getConnection(row).getPort();
        	case RCVDMSGS:
        		return new Integer(getConnection(row).getRecvdMsgs());
        	case REGEXP:
        		return getConnection(row).getRegexp();
        	case DEVICE:
        		return getConnection(row).getDevice();
        	case FILTER:
        		return getConnection(row).getFilter();
        	case CONNECTED:
        		return getConnection(row).getConnected();
            case USERNAME:
               return getConnection(row).getUsername();
            case PASSWORD:
               return "***";
        	default:
        		return "Bad col: " + column;
        }
    }
    
	public Connection getConnection(int i) {
        return (Connection) connections.getConnection(i);
    }

	public void connectionsEvent(ConnectionsEvent e) {
		if(e.getEventType()==ConnectionManager.ADDED){
			fireTableRowsInserted(0,connections.numConnections());
		}else if(e.getEventType()==ConnectionManager.REMOVED){
			fireTableRowsDeleted(0,connections.numConnections());
		}
	}
	
    public boolean isCellEditable(int row, int col) {
        if (col != 0) {
            return true;
        } else {
            return false;
        }
    }
    
    public void setValueAt(Object value, int row, int col) {
    	log.debug("setting value:"+value);
    	Connection c = getConnection(row);
    	switch(col){
    	case ICON:
    		return;
    	case IP:
    		c.setIp((String)value);
    		break;
    	case PORT:
    		c.setPort((String)value);
    		break;
    	case RCVDMSGS:
    		return;
    	case REGEXP:
    		c.setRegexp((String)value);
    		break;
    	case DEVICE:
    		c.setDevice((String)value);
    		break;
    	case FILTER:
    		c.setFilter((String)value);
    		break;
    	case CONNECTED:
            try {
					c.setConnected((Boolean)value);
				} catch (Exception e) {
					error(e.getMessage(),e.getClass().getName());
				}
    		break;
        case USERNAME:
           c.setUsername((String)value);
           break;
        case PASSWORD:
           c.setPassword((String)value);
           break;
        default:
            return;	
    	}
        fireTableCellUpdated(row, col);
    }
    
    private void error(String msg, String title) {
        JOptionPane.showMessageDialog(this.table, msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }
}
