/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package com.voztele.sipspy.connections;

import java.util.HashMap;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

import com.voztele.sipspy.SipMessage;

public abstract class DataHandler implements ContentHandler {
	protected String from;

	protected String to;

	protected String fromPort;

	protected String toPort;

	protected String time;

	protected String typeS;

	protected int type;

	protected String statusMessage;

	protected String transactionId;

	protected String firstLine;

	protected String firstViaHost;

	protected String firstViaPort;

	protected String callId;

	protected String debugLine;

	protected StringBuffer message;

	protected String currentTag;

	protected Attributes responseAttrs;
    
    protected HashMap commands = new HashMap();

	private static Logger log = Logger.getLogger(DataHandler.class);

	public void setDocumentLocator(Locator arg0) {
	}

	public void startDocument() throws SAXException {
	}

	public void endDocument() throws SAXException {
	}

	public void startPrefixMapping(String arg0, String arg1)
			throws SAXException {
	}

	public void endPrefixMapping(String arg0) throws SAXException {
	}

	public synchronized void startElement(String namespaceURI, String lName,
			String qName, Attributes attrs) throws SAXException {
		currentTag = qName;
		if (qName.equalsIgnoreCase("message")) {
			from = attrs.getValue("from");
			to = attrs.getValue("to");
			fromPort = attrs.getValue("fromPort");
			toPort = attrs.getValue("toPort");
			time = attrs.getValue("time");
			typeS = attrs.getValue("type");
			if (typeS != null) {
				type = Integer.parseInt(typeS);
			}
			statusMessage = attrs.getValue("statusMessage");
			transactionId = attrs.getValue("transactionId");
			firstLine = attrs.getValue("firstLine");
			firstViaHost = attrs.getValue("firstViaHost");
			firstViaPort = attrs.getValue("firstViaPort");
			callId = attrs.getValue("callId");
			debugLine = attrs.getValue("debugLine");
			message = new StringBuffer();
		} else if (qName.equalsIgnoreCase("response")) {
			log.debug("response rcvd");
			responseAttrs = new AttributesImpl(attrs);
		} else if (qName.equalsIgnoreCase("file")) {
            log.debug("file rcvd:"+attrs.getValue("name"));
            this.addFile(attrs.getValue("name"));
        } else if (qName.equals("device")) {
			addDevice(attrs.getValue("name"), attrs.getValue("ip"));
		}
	}

	public synchronized void endElement(String namespaceURI, String sName, String qName)
			throws SAXException {
		if (qName.equalsIgnoreCase("message")) {
			SipMessage sm = new SipMessage(type, from, to, fromPort, toPort,
					time, firstLine, firstViaHost, firstViaPort, message
							.toString(), statusMessage, transactionId,
					debugLine, callId);
			processMessage(sm);
		} else if (qName.equalsIgnoreCase("response")) {
           log.debug("end response");
           notifyAll();
		} else if (qName.equals("device")) {
		}
	}

	public void characters(char buf[], int offset, int len) throws SAXException {
		if (buf == null)
			return;
		if (currentTag.equalsIgnoreCase("message")) {
			String str = new String(buf, offset, len);
			if (str.equals(""))
				return;
			message.append(str);
		}
	}

	public void ignorableWhitespace(char[] arg0, int arg1, int arg2)
			throws SAXException {
	}

	public void processingInstruction(String arg0, String arg1)
			throws SAXException {
	}

	public void skippedEntity(String arg0) throws SAXException {
	}

	public abstract void addDevice(String device, String ip);

	public abstract void processMessage(SipMessage msg);
    
    public abstract void addFile(String file);

}
