/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * Created on 01/09/2005 by Voztele.com
 *
 * TODO Add javadoc, edit members ..
 * 
 */
package com.voztele.sipspy.connections;

import java.io.IOException;
import java.io.OutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.ArrayList;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;

import com.voztele.sipspy.MessageListener;
import com.voztele.sipspy.SipMessage;

public class DataThread extends DataHandler implements Runnable,ErrorHandler {
	
	private Connection connection;
	private Socket socket;
	private XMLReader xmlReader;
	private InputSource inputSource = null;
    private ArrayList messageListeners = null;
    private int cseq=0;

    private static Logger log=Logger.getLogger(DataThread.class);
    private Thread myThread;
	
	public DataThread(Connection c,Socket s) throws ParserConfigurationException, SAXException, IOException {
		this.connection = c;
		SAXParserFactory saxParserFactory=SAXParserFactory.newInstance();
		SAXParser saxParser = saxParserFactory.newSAXParser();
        this.messageListeners=c.getMessageListeners();
		this.xmlReader = saxParser.getXMLReader();
		this.socket = s;
		socket.setTcpNoDelay(true);
		socket.setReceiveBufferSize(20);
		xmlReader.setContentHandler(this);
		xmlReader.setFeature("http://xml.org/sax/features/validation",false);
		this.inputSource = new InputSource(socket.getInputStream());
        myThread = new Thread(this);
		myThread.start();
	}

	/**Method that keeps on reading from the input stream.
	 * 
	 */
	public void run(){
		try {
			xmlReader.parse(inputSource);
		}catch(IOException ioe){
		}catch (SAXException e) {
			this.connection.error(e.getMessage(),e.getClass().getName());
			try {
				connection.setConnected(new Boolean(false));
			} catch (UnknownHostException e3) {
			} catch (IOException e2) {
			}
		}
	}

	
	public synchronized Attributes sendCommand(String command,String arg) throws IOException{
       log.debug("Sending command:"+command+" arg="+arg+" cseq= "+cseq++);
       byte arrai[] = arg.getBytes();
		this.socket.getOutputStream().write((command+"\n"+cseq+"\n"+arrai.length+"\n").getBytes());
        this.socket.getOutputStream().write(arrai);
		this.socket.getOutputStream().flush();
		try {
			log.debug("beforeAwait");
			wait(6000);//FIXME this should be configurable
		} catch (InterruptedException e) {
			log.debug("waitinterrupted");
			return null;
		}
        if(responseAttrs==null)
           return null;
          if (log.isDebugEnabled())
             for (int i = 0; i < responseAttrs.getLength(); i++)
                log.debug("result." + responseAttrs.getQName(i) + "=" + responseAttrs.getValue(i));
		return responseAttrs;
	}
    
    public synchronized void sendArray(byte[] arrai) throws IOException{
        this.socket.getOutputStream().write(arrai);
        this.socket.getOutputStream().flush();
    }
    
    public void addDevice(String device,String ip){
       connection.addDevice(device,ip);
    }

   public void processMessage(SipMessage msg) {
      for(int i=0;i<messageListeners.size();i++){
         ((MessageListener) messageListeners.get(i)).messageEvent(msg);
     }
   }

	public void warning(SAXParseException e) throws SAXException {
		this.connection.error(e.getMessage(),e.getClass().getName());
	}
	
	public void error(SAXParseException e) throws SAXException {
		this.connection.error(e.getMessage(),e.getClass().getName());
        notifyAll();
	}
	
	public void fatalError(SAXParseException e) throws SAXException {
       notifyAll();
		this.connection.error(e.getMessage(),e.getClass().getName());
	}

   public void addFile(String file) {
      connection.addSessionElem(file);
   }
}
