/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package com.voztele.sipspy.connections;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Element;


public class NewConnectionFrame extends JFrame implements DocumentListener,ActionListener{
	
	private static final long serialVersionUID = -1659125974468115475L;
	private Connection connection;
	private JTextField ipTextField;
	private JTextField portTextField;
	private JTextField regexpTextField;
	private JTextField deviceTextField;
	private JTextField filterTextField;
    private JTextField usernameTextField;
    private JPasswordField passwordField;
    
	public NewConnectionFrame(Connection c,ConnectionsFrame cfs) throws ParserConfigurationException{
		super("New Connection");
		this.connection = c;
		
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                setVisible(false);
                dispose();
            }
        });
        
        int fontSize=12;
        String fontName="Helvetica";
        int fontType=Font.BOLD;
        
        JPanel contentPane = new JPanel();
        contentPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        setContentPane(contentPane);
        
        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints ct = new GridBagConstraints();
        contentPane.setLayout(gridBag);
		
        ct.gridwidth = GridBagConstraints.RELATIVE;
        ct.weightx = 1.0;
        ct.anchor = GridBagConstraints.NORTHWEST;
        ct.fill = GridBagConstraints.BOTH;
        //****************** The IP label
		JLabel descriptionLabel = new JLabel("Enter SpyAgent IP");
		descriptionLabel.setFont(new Font(fontName, fontType, fontSize));
		gridBag.setConstraints(descriptionLabel,ct);
		contentPane.add(descriptionLabel);
		//The IP text Field
		ipTextField = new JTextField( 30 );
		ipTextField.getDocument().putProperty("attributeName","ip");
		ct.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(ipTextField,ct);
		contentPane.add(ipTextField);
		ipTextField.getDocument().addDocumentListener(this);
        //****************** The port label
		JLabel descriptionLabel2 = new JLabel("Enter SpyAgent port");
		descriptionLabel2.setFont(new Font(fontName, fontType, fontSize));
		ct.gridwidth = GridBagConstraints.RELATIVE;
		gridBag.setConstraints(descriptionLabel2,ct);
		contentPane.add(descriptionLabel2);
		//The port text Field		
		portTextField = new JTextField( "23332" );
		portTextField.getDocument().putProperty("attributeName","port");
		ct.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(portTextField,ct);
		portTextField.getDocument().addDocumentListener(this);
		contentPane.add(portTextField);
        //****************** The regexp label		
		JLabel descriptionLabel3 = new JLabel("Enter Regular Expression");
		descriptionLabel3.setFont(new Font(fontName, fontType, fontSize));
		ct.gridwidth = GridBagConstraints.RELATIVE;
		gridBag.setConstraints(descriptionLabel3,ct);
		contentPane.add(descriptionLabel3);
		//The regexp text Field		
		regexpTextField = new JTextField( "sip:.*" );
		regexpTextField.getDocument().putProperty("attributeName","regexp");
		ct.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(regexpTextField,ct);
		regexpTextField.getDocument().addDocumentListener(this);
		contentPane.add(regexpTextField);
        //****************** The device label		
		JLabel descriptionLabel4 = new JLabel("Enter device to capture in");
		descriptionLabel4.setFont(new Font(fontName, fontType, fontSize));
		ct.gridwidth = GridBagConstraints.RELATIVE;
		gridBag.setConstraints(descriptionLabel4,ct);
		contentPane.add(descriptionLabel4);
		//The device text Field		
		deviceTextField = new JTextField( "eth0" );
		deviceTextField.getDocument().putProperty("attributeName","device");
		ct.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(deviceTextField,ct);
		deviceTextField.getDocument().addDocumentListener(this);
		contentPane.add(deviceTextField);
        //****************** The capture filter label		
		JLabel descriptionLabel5 = new JLabel("Enter the capture filter");
		descriptionLabel5.setFont(new Font(fontName, fontType, fontSize));
		ct.gridwidth = GridBagConstraints.RELATIVE;
		gridBag.setConstraints(descriptionLabel5,ct);
		contentPane.add(descriptionLabel5);
		//The Filter text Field		
		filterTextField = new JTextField( "port 5060" );
		filterTextField.getDocument().putProperty("attributeName","filter");
		ct.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(filterTextField,ct);
		filterTextField.getDocument().addDocumentListener(this);
		contentPane.add(filterTextField);
        //****************** The Username label       
        JLabel descriptionLabel6 = new JLabel("Enter the username");
        descriptionLabel5.setFont(new Font(fontName, fontType, fontSize));
        ct.gridwidth = GridBagConstraints.RELATIVE;
        gridBag.setConstraints(descriptionLabel6,ct);
        contentPane.add(descriptionLabel6);
        //The username Field     
        usernameTextField = new JTextField( "" );
        usernameTextField.getDocument().putProperty("attributeName","username");
        ct.gridwidth = GridBagConstraints.REMAINDER;
        gridBag.setConstraints(usernameTextField,ct);
        usernameTextField.getDocument().addDocumentListener(this);
        contentPane.add(usernameTextField);
        //****************** The Password label       
        JLabel descriptionLabel7 = new JLabel("Enter the password");
        descriptionLabel5.setFont(new Font(fontName, fontType, fontSize));
        ct.gridwidth = GridBagConstraints.RELATIVE;
        gridBag.setConstraints(descriptionLabel7,ct);
        contentPane.add(descriptionLabel7);
        //The password Field     
        passwordField = new JPasswordField();
        passwordField.getDocument().putProperty("attributeName","password");
        ct.gridwidth = GridBagConstraints.REMAINDER;
        gridBag.setConstraints(passwordField,ct);
        passwordField.getDocument().addDocumentListener(this);
        contentPane.add(passwordField);
		
		Element ele = connection.addConfigElement("port","23332");
		this.connection.getConfigElm().appendChild(ele);
		ele = connection.addConfigElement("regexp","sip:.*@voztele.com");
		this.connection.getConfigElm().appendChild(ele);
		ele = connection.addConfigElement("device","eth0");
		this.connection.getConfigElm().appendChild(ele);
		ele = connection.addConfigElement("filter","src port 5060 or dst port 5060");
		this.connection.getConfigElm().appendChild(ele);
		ele = connection.addConfigElement("ip","");
		this.connection.getConfigElm().appendChild(ele);
        ele = connection.addConfigElement("username","");
        this.connection.getConfigElm().appendChild(ele);
        ele = connection.addConfigElement("password","");
        this.connection.getConfigElm().appendChild(ele);
		
		ct.gridwidth = GridBagConstraints.REMAINDER;
		JButton botoncito = new JButton("ok");
		gridBag.setConstraints(botoncito,ct);
		botoncito.addActionListener(this);
		contentPane.add(botoncito);
		
		setSize(400,200);
		setLocation(Toolkit.getDefaultToolkit().getScreenSize().width/2,
				Toolkit.getDefaultToolkit().getScreenSize().height/2);
		setVisible(true);
	}

	public void insertUpdate(DocumentEvent arg0) {
		changedUpdate(arg0);
	}

	public void removeUpdate(DocumentEvent arg0) {
		changedUpdate(arg0);				
	}

	public void changedUpdate(DocumentEvent arg0) {
		Element ele = connection.getConfigElm();
		String atributo = (String) arg0.getDocument().getProperty("attributeName");
		JTextField jtf = null;
		if(atributo.equals("ip")){
			connection.setIp(ipTextField.getText());
		}else if(atributo.equals("port")){
			connection.setPort(portTextField.getText());
		}else if(atributo.equals("regexp")){
			connection.setRegexp(regexpTextField.getText());
		}else if(atributo.equals("device")){
			connection.setDevice(deviceTextField.getText());
		}else if(atributo.equals("filter")){
			connection.setFilter(filterTextField.getText());
		}else if(atributo.equals("username")){
            connection.setUsername(usernameTextField.getText());
        }else if(atributo.equals("password")){
            connection.setPassword(new String(passwordField.getPassword()));
        }
	}

	public void actionPerformed(ActionEvent arg0) {
		if(	connection.getIp()==null || 
			connection.getPort()==null ||
			connection.getIp().equals("")||
			connection.getPort().equals("")){
			JOptionPane.showMessageDialog(this,"You should set the IP, PORT and REGEXP elements before saying OK", "ERROR",
                    JOptionPane.ERROR_MESSAGE);
    	}else{
    		connection.add();
    	}
        setVisible(false);
        dispose();
	}
	
}
