/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package com.voztele.sipspy.connections;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.log4j.Logger;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import com.voztele.sipspy.LogComparator;
import com.voztele.sipspy.SipMessage;
import com.voztele.sipspy.SpySession;
import com.voztele.sipspy.SpySessionListener;
import com.voztele.sipspy.SpySessionProvider;

public class SessionFileParser extends DataHandler implements SpySessionProvider{
	
	private Logger log = Logger.getLogger(this.getClass());
	private Hashtable sessions = new Hashtable();
	private ArrayList spySessionListeners = new ArrayList();
	private XMLReader xmlReader;
	
	public SessionFileParser() throws ParserConfigurationException, SAXException{
		SAXParserFactory saxParserFactory=SAXParserFactory.newInstance();
		SAXParser saxParser = saxParserFactory.newSAXParser();
		this.xmlReader = saxParser.getXMLReader();
		xmlReader.setContentHandler(this);
		xmlReader.setFeature("http://xml.org/sax/features/validation",false);
	}
	
	public void addSessionsListener(SpySessionListener ml) {
		this.spySessionListeners.add(ml);
	}
	
	public void removeSessionsListener(SpySessionListener ml) {
		this.spySessionListeners.remove(ml);
	}
	
	public ArrayList getSessionsListeners() {
		return spySessionListeners;
	}
	
	private void fireSessionEvent(SpySession ss) {
		for(int i=0;i<spySessionListeners.size();i++){
			((SpySessionListener)spySessionListeners.get(i)).sessionEvent(ss);
		}
	}
	
	public void parse(InputStream is) throws IOException, SAXException{
		xmlReader.parse(new InputSource(is));
	}

   public void addDevice(String device, String ip) {
   }

   public void processMessage(SipMessage msg) {
        log.debug("Message event:" + firstLine);
        SpySession mySession=(SpySession) sessions.get(callId);
        if (mySession == null){
            mySession = new SpySession(callId,new LogComparator());
            sessions.put(callId, mySession);
            mySession.add(msg);
            fireSessionEvent(mySession);
        }else{
            mySession.add(msg);
        }

      
   }

   public void addFile(String file) {
      
   }
}
