/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package com.voztele.sipspy.flowpanel;

import java.awt.Color;
import java.awt.Graphics;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;

import com.voztele.sipspy.SipMessage;

public abstract class Arrow {

	public String arrowName;
	public SipMessage sipMessage;

	public boolean visible = true;
	public boolean selected;

	public Color color;

	public int xmin;
	public int xmax;
	public int ymin;
	public int ymax;

	abstract int xmin();
	abstract int xmax();
	abstract int ymin();
	abstract int ymax();

	public int xminInfo;
	public int xmaxInfo;
	public int yminInfo;
	public int ymaxInfo;
    
	protected GregorianCalendar gc = new GregorianCalendar();
    protected SimpleDateFormat sdf = new SimpleDateFormat("dd/MMM/yyyy-HH:mm:ss.S");

	abstract void draw(Graphics g);

	public void setColor(Color color) {
		this.color = color;
	}

	public void setSipMessage(SipMessage tracesMessage) {
		this.sipMessage = tracesMessage;
	}

	public SipMessage getSipMessage() {
		return sipMessage;
	}

	public Arrow(boolean selected,String arrowName,boolean flag,int xmin,int xmax,int ymin,int ymax) {
		this.arrowName = arrowName;
		this.xmin = xmin;
		this.xmax = xmax;
		this.ymin = ymin;
		this.ymax = ymax;
		this.selected = selected;
		visible = flag;
	}

	public boolean isCollisionArrow(int x, int y) {
		if (xmin <= xmax) {
			if (x < xmax && x > xmin)
				if (y < ymax && y > ymin)
					return true;
				else
					return false;
			else
				return false;
		} else {
			if (x < xmin && x > xmax)
				if (y < ymax && y > ymin)
					return true;
				else
					return false;
			else
				return false;
		}
	}

	public boolean isCollisionInfo(int x, int y) {
		if (x < xmaxInfo && x > xminInfo)
			if (y < ymaxInfo && y > yminInfo)
				return true;
			else
				return false;
		else
			return false;
	}

}
