/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/* This class displays the trace logged by the server
 * Here is a description of the properties
 *
 *   -------------------------------------------------------  ^
 *   |                                                     |  |
 *   |       Actor 1          Actor 2         Actor 3      |  | 3
 *   |							   |  |
 *   |------------------------------------------------------  v ^
 *   |           |               |               |         |    | 4
 *   |           |-------------->|               |         |    v
 *   |           |               |               |         |
 *   |           |               |               |         |
 *   |           |               |-------------->|         |  ^  
 *   |           |               |               |         |  | 5
 *   |           |               |               |         |  |
 *   |           |               |<--------------|         |  v
 *   |           |               |               |         |
 *   |           |               |               |         |
 *   |           |<--------------|               |         |  ^
 *   |           |               |               |         |  | 6
 *   -------------------------------------------------------  v
 *                                                    7
 *         1                             2       <--------->
 *   <----------->               <--------------->
 *
 *
 *  1 : FIRST_ACTOR_GAP
 *  2 : HORIZONTAL_GAP
 *  3 : ACTORS_STRIPE
 *  4 : FIRST_ARROW_GAP
 *  5 : VERTICAL_GAP
 *  6 : LAST_ARROW_GAP
 *  7 : LAST_ACTOR_GAP
 */

package com.voztele.sipspy.flowpanel;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.Scrollable;
import javax.swing.SwingConstants;

import org.apache.log4j.Logger;

import com.voztele.sipspy.LogComparator;
import com.voztele.sipspy.MessageListener;
import com.voztele.sipspy.SipMessage;
import com.voztele.sipspy.SipSpy;
import com.voztele.sipspy.SpySession;
import com.voztele.sipspy.Utils;

public class StaticTracesCanvas extends JPanel implements MouseListener, MessageListener,
      Scrollable {

   private Logger log = Logger.getLogger(this.getClass());

   private JScrollPane scrollPane = null;
   
   private boolean strictAddresses = false;

   public static int FIRST_ACTOR_GAP = 100;

   public static int HORIZONTAL_GAP = 200;

   public static int ACTORS_STRIPE = 120;

   public static int FIRST_ARROW_GAP = 50;

   public static int VERTICAL_GAP = 60;

   public static int LAST_ARROW_GAP = 30;

   public static int LAST_ACTOR_GAP = 100;

   // This color is the color of the current selected SIP message
   public Color SELECTED_COLOR = Color.red;

   public Color[] ARROWS_COLOR = { Color.blue, colorize(Color.blue, +50),
         colorize(Color.blue, +100), colorize(Color.green, -50), Color.green,
         colorize(Color.green, +50), colorize(Color.orange, -50), Color.orange,
         colorize(Color.orange, +50), colorize(Color.magenta, -75), Color.magenta,
         colorize(Color.magenta, +75), colorize(Color.cyan, -150), colorize(Color.cyan, -75),
         Color.cyan, };

   public String selectedArrowName;

   public Arrow newArrow;

   public Arrow oldArrow;

   public Image actorsImage;

   public Image backgroundImage;

   public SpySession[] spySessionList;

   public JTextArea messageContentTextArea = null;

   public String tracesOrigin;

   public Hashtable actors = new Hashtable();

   public Hashtable arrows = new Hashtable();

   public Hashtable arrowsColors = new Hashtable();
   
   public Hashtable portsBinds = new Hashtable();

   public Arrow arrowTipTool = null;

   public SipSpy sipSpy;

   public SipMessage debugTracesMessage;

   private int maxUnitIncrement = 15;
   
   private Image logo = Utils.getImage("voztele_logo.jpg");

   public Dimension getPreferredSize() {
	   if(actors.isEmpty() && arrows.isEmpty()){
		   return new Dimension(231,58);
	   }
	   int width = FIRST_ACTOR_GAP + LAST_ACTOR_GAP + HORIZONTAL_GAP * (actors.size() - 1);
	   int height = ACTORS_STRIPE + FIRST_ARROW_GAP + LAST_ARROW_GAP + VERTICAL_GAP * (arrows.size());
	   return new Dimension(width <= 0 ? 0 : width, height);
   }

   private Color colorize(Color c, int i) {
      return new Color((c.getRed() + i) < 0 ? 0 : (c.getRed() + i) > 255 ? 255 : (c.getRed() + i),
            (c.getGreen() + i) < 0 ? 0 : (c.getGreen() + i) > 255 ? 255 : (c.getGreen() + i), (c
                  .getBlue() + i) < 0 ? 0 : (c.getBlue() + i) > 255 ? 255 : (c.getBlue() + i));
   }

   public StaticTracesCanvas(JTextArea messageContentTextArea2, String string, SipSpy spy) {
      this.sipSpy = spy;
      backgroundImage = Utils.getImage("back.gif");
      actorsImage = Utils.getImage("computer4.gif");
      String sa=null;
      if(!sipSpy.isApplet())
    	  sa =  System.getProperty("strict.addresses");
      else
    	  sa = sipSpy.getParameter("strict.addresses");
      this.strictAddresses = (sa != null && sa.equals("true")) ? true : false;
      this.messageContentTextArea = messageContentTextArea2;
      addMouseListener(this);
   }

   public void refreshTracesCanvas(Object[] sessions) {
      if (sessions.length == 0)
         return;
      int i;
      if (spySessionList != null)
         for (i = 0; i < this.spySessionList.length; i++)
            spySessionList[i].removeMessageListener(this);

      this.spySessionList = new SpySession[sessions.length];

      for (i = 0; i < sessions.length; i++) {
         spySessionList[i] = (SpySession) sessions[i];
         spySessionList[i].addMessageListener(this);
      }

      log.debug("painting " + i + " sessions");
      constructActors();
      constructArrows();
      setVisible(true);
      selectMessage(FIRST_ACTOR_GAP + HORIZONTAL_GAP / 2, ACTORS_STRIPE + FIRST_ARROW_GAP);
      repaint();
   }

   public void drawTop(Graphics g) {

      int widthCanvas = getPreferredSize().width;
      int heightCanvas = getPreferredSize().height;

      g.setColor(new Color(0, 0, 125));
      g.fillRect(0, 0, widthCanvas, ACTORS_STRIPE - 5);
      // Draw the String using the current Font and color
      g.setColor(Color.white);
      // display the label
      Font f = g.getFont();
      Font newFont = new Font(f.getName(), Font.BOLD | Font.ITALIC, 17);
      g.setFont(newFont);
      // draw a separation line:
      g.setColor(Color.black);
      // x,y -> x,y
      g.drawLine(0, ACTORS_STRIPE, widthCanvas, ACTORS_STRIPE);

      // draw the actors above the separation line and their vertical line:
      Enumeration e = actors.keys();
      while (e.hasMoreElements()) {
         String origin = (String) e.nextElement();
         int positionX = ((Integer) actors.get(origin)).intValue();
         g.setColor(Color.white);
         f = g.getFont();
         newFont = new Font(f.getName(), Font.BOLD | Font.ITALIC, 14);
         g.setFont(newFont);
         g.drawString(origin, positionX - getFontMetrics(g.getFont()).stringWidth(origin) / 2,
               ACTORS_STRIPE - 15);
         if (actorsImage != null) {
            g.drawImage(actorsImage, positionX - actorsImage.getWidth(this) / 2, ACTORS_STRIPE / 5,
                  this);
         } else {
         }
         g.setColor(Color.black);
         g.drawLine(positionX, ACTORS_STRIPE, positionX, heightCanvas);
      }
   }
   public void paintLogo(Graphics g) {
	   g.drawImage(logo,0,0,null);
   }

   
   public void constructActors() {
      SpySession spySession;
      try {
         // We clean the table
         actors = new Hashtable();
         portsBinds = new Hashtable();
         
         for (int i = 0; i < spySessionList.length; i++) {
            spySession = spySessionList[i];
            Iterator ite = spySession.iterator();
            while (ite.hasNext()) {
               SipMessage tracesMessage = (SipMessage) ite.next();
               String from, to;
               // We have to stock the actor and its position in the canvas
               int sizeActors = actors.size();
               from = getFrom(tracesMessage);
               to=getTo(tracesMessage);
               if (actors.get(from) == null) {
                  actors.put(from, new Integer(sizeActors * HORIZONTAL_GAP + FIRST_ACTOR_GAP));
               }
               sizeActors = actors.size();
               if (actors.get(to) == null) {
                  actors.put(to, new Integer(sizeActors * HORIZONTAL_GAP + FIRST_ACTOR_GAP));
               }
            }
         }
      } catch (Exception e) {
         System.out.println("Error in trying to construct the actors");
      }
   }

   public void constructArrows() {
      arrows = new Hashtable();
      String from = null;
      String to = null;
      int i = 0;
      SpySession spySession;

      // Assign different colors to each arrows
      assignColors();
      // System.out.println("Construct arrows");

      SpySession sp;
      // Setup the first selected arrow:
      selectedArrowName = "arrow0";
      if (spySessionList.length > 1) {
         sp = new SpySession("fakeID", new LogComparator());
         for (int j = 0; j < spySessionList.length; j++) {
            spySession = (SpySession) spySessionList[j];
            Iterator ite = spySession.iterator();
            while (ite.hasNext()) {
               SipMessage message = (SipMessage) ite.next();
               sp.add(message);
            }
         }
      } else
         sp = spySessionList[0];
      Iterator ite = sp.iterator();
      while (ite.hasNext()) {
         SipMessage message = (SipMessage) ite.next();
         from = getFrom(message);
         to = getTo(message);
         int positionXFrom = ((Integer) actors.get(from)).intValue();
         int positionXTo = ((Integer) actors.get(to)).intValue();
         int positionY = i * VERTICAL_GAP + ACTORS_STRIPE + FIRST_ARROW_GAP;

         String arrowName = "arrow" + i;
         boolean selected = false;

         // Set up the color of this arrow
         String transactionId = message.getTransactionId();

         Color color;
         if (transactionId != null) {
            color = (Color) arrowsColors.get(transactionId);
            if (color == null)
               color = Color.black;
         } else
            color = Color.black;

         if (positionXFrom == positionXTo) {
            // This is a loop !!
            CircleArrow circleArrow = new CircleArrow(selected, arrowName, positionXFrom,
                  positionY - 20, positionY + 20, 40, true);
            circleArrow.setColor(color);
            circleArrow.setSipMessage(message);
            arrows.put(arrowName, circleArrow);
         } else {
            // This is a straight arrow
            StraightArrow straightArrow = new StraightArrow(selected, arrowName, positionXFrom,
                  positionXTo, positionY - 31, positionY + 31, true);
            straightArrow.setColor(color);
            straightArrow.setSipMessage(message);
            arrows.put(arrowName, straightArrow);
         }
         i++;
      }
   }// for

   public void assignColors() {
      arrowsColors = new Hashtable();
      SipMessage tracesMessage = null;
      SpySession spySession;
      int colorIndex = 0;
      for (int i = 0; i < spySessionList.length; i++) {
         spySession = (SpySession) spySessionList[i];
         Iterator ite = spySession.iterator();
         while (ite.hasNext()) {
            tracesMessage = (SipMessage) ite.next();
            String transactionId = tracesMessage.getTransactionId();

            if (transactionId != null) {
               Color color = (Color) arrowsColors.get(transactionId);
               if (color == null) {
                  color = ARROWS_COLOR[colorIndex % ARROWS_COLOR.length];
                  arrowsColors.put(transactionId, color);
                  colorIndex++;
               }
            }
         }
      }
   }

   public Arrow getArrow(int x, int y) {
      Enumeration e = arrows.keys();
      while (e.hasMoreElements()) {
         String arrowName = (String) e.nextElement();
         Arrow arrow = (Arrow) arrows.get(arrowName);
         if (arrow == null)
            return null;
         if (arrow.isCollisionArrow(x, y))
            return arrow;
      }
      return null;
   }

   public void paintComponent(Graphics g) {
      g.clearRect(0, 0, getPreferredSize().width, getPreferredSize().height);
	   if(actors.isEmpty() && arrows.isEmpty()){
		   paintLogo(g);
		   return;
	   }else{
		   drawTop(g);
		   drawArrows(g);
	   }
   }

   public void selectMessage(int x, int y) {
      newArrow = getArrow(x, y);
      if (newArrow == null) {

      } else {
         // We have to update the content:
         SipMessage tracesMessage = newArrow.getSipMessage();
         if (messageContentTextArea != null) {
            messageContentTextArea.setText(tracesMessage.getMessageString());
         }
         debugTracesMessage = tracesMessage;

         // We have to change the color of the old message and the new one:
         oldArrow = (Arrow) arrows.get(selectedArrowName);
         oldArrow.selected = false;
         newArrow.selected = true;
         selectedArrowName = newArrow.arrowName;
      }
   }

   public void drawArrows(Graphics g) {
      Enumeration e = arrows.keys();
      while (e.hasMoreElements()) {
         String arrowName = (String) e.nextElement();
         Arrow arrow = (Arrow) arrows.get(arrowName);
         if (arrow != null) {
            arrow.draw(g);
         }
      }
      if (getParent() != null) {
         getParent().doLayout();
         getParent().validate();
      }
   }

   public void unselectAllArrows() {
      Enumeration e = arrows.keys();
      while (e.hasMoreElements()) {
         String arrowName = (String) e.nextElement();
         Arrow arrow = (Arrow) arrows.get(arrowName);
         arrow.selected = false;
      }
   }

   public boolean isOnArrow(int x, int y) {
      Arrow arrow = getArrow(x, y);
      if (arrow == null)
         return false;
      else
         return true;
   }

   public void mousePressed(MouseEvent e) {
      try {
         int x = e.getX();
         int y = e.getY();

         if (isOnArrow(x, y)) {
            selectMessage(x, y);
            repaint();
         }
      } catch (Exception ex) {
      }
   }

   public void mouseReleased(MouseEvent p1) {
   }

   public void mouseEntered(MouseEvent p1) {
   }

   public void mouseClicked(MouseEvent p1) {
   }

   public void mouseExited(MouseEvent p1) {
   }

   public void messageEvent(SipMessage tm) {
      Dimension d = getPreferredSize();
      int x = 0;
      int y = d.height - 600;
      Rectangle rect = new Rectangle(x, y, getSize().width, getSize().height);
      scrollPane.scrollRectToVisible(rect);
      constructActors();
      constructArrows();
      setVisible(true);
      selectMessage(FIRST_ACTOR_GAP + HORIZONTAL_GAP / 2, ACTORS_STRIPE + FIRST_ARROW_GAP);
      revalidate();
      repaint();
   }

   public void addArrow(SipMessage msg) {

      String fromT = msg.getFrom();
      String toT = msg.getTo();

      int sizeActors = actors.size();
      if (actors.get(fromT) == null) {
         actors.put(fromT, new Integer(sizeActors * HORIZONTAL_GAP + FIRST_ACTOR_GAP));
      }
      sizeActors = actors.size();
      if (actors.get(toT) == null) {
         actors.put(toT, new Integer(sizeActors * HORIZONTAL_GAP + FIRST_ACTOR_GAP));
      }

      String from = msg.getFrom();
      String to = msg.getTo();

      int positionXFrom = ((Integer) actors.get(from)).intValue();
      int positionXTo = ((Integer) actors.get(to)).intValue();
      int positionY = arrows.size() * VERTICAL_GAP + ACTORS_STRIPE + FIRST_ARROW_GAP;

      String arrowName = "arrow" + (arrows.size() + 1);
      boolean selected = false;

      // Set up the color of this arrow
      String transactionId = msg.getTransactionId();

      Color color;
      if (transactionId != null) {
         color = (Color) arrowsColors.get(transactionId);
         if (color == null)
            color = Color.black;
      } else
         color = Color.black;

      if (positionXFrom == positionXTo) {
         CircleArrow circleArrow = new CircleArrow(selected, arrowName, positionXFrom,
               positionY - 20, positionY + 20, 40, true);
         circleArrow.setColor(color);
         circleArrow.setSipMessage(msg);
         arrows.put(arrowName, circleArrow);
      } else {
         StraightArrow straightArrow = new StraightArrow(selected, arrowName, positionXFrom,
               positionXTo, positionY - 31, positionY + 31, true);
         straightArrow.setColor(color);
         straightArrow.setSipMessage(msg);
         arrows.put(arrowName, straightArrow);
      }
   }

   public void finishDocument() {

   }

   public Dimension getPreferredScrollableViewportSize() {
      return getPreferredSize();
   }

   public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
      //Get the current position.
      int currentPosition = 0;
      if (orientation == SwingConstants.HORIZONTAL) {
         currentPosition = visibleRect.x;
      } else {
         currentPosition = visibleRect.y;
      }

      if (direction < 0) {
         int newPosition = currentPosition - (currentPosition / maxUnitIncrement)
               * maxUnitIncrement;
         return (newPosition == 0) ? maxUnitIncrement : newPosition;
      } else {
         return ((currentPosition / maxUnitIncrement) + 1) * maxUnitIncrement - currentPosition;
      }
   }

   public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
      if (orientation == SwingConstants.HORIZONTAL) {
         return visibleRect.width - maxUnitIncrement;
      } else {
         return visibleRect.height - maxUnitIncrement;
      }
   }

   public boolean getScrollableTracksViewportWidth() {
      return false;
   }

   public boolean getScrollableTracksViewportHeight() {
      return false;
   }

   public void setMaxUnitIncrement(int pixels) {
      maxUnitIncrement = pixels;
   }

   /**
    * @return Returns the scrollPane.
    */
   public JScrollPane getScrollPane() {
      return scrollPane;
   }

   /**
    * @param scrollPane The scrollPane to set.
    */
   public void setScrollPane(JScrollPane scrollPane) {
      this.scrollPane = scrollPane;
   }
   
   public String getFrom(SipMessage msg){
	   if(strictAddresses){
		   return msg.getFrom() + ":" + msg.getFromPort();
	   }
      if (msg.isRequest()){
         int srcport=Integer.parseInt(msg.getFromPort());
         if(srcport>=32768)
            portsBinds.put(new Integer(srcport),msg.getFirstViaPort());
         return msg.getFirstViaHost()+":"+msg.getFirstViaPort();
      }else {
         String port =msg.getFromPort();
         int fromport =Integer.parseInt(port);
         if(fromport>=32768)
            port=(String) portsBinds.get(new Integer(fromport));
         if(port==null){
            port=msg.getFromPort();
         }
         return msg.getFrom()+":"+port;
       }
   }
   
   public String getTo(SipMessage msg){
	   if(strictAddresses)
		   return msg.getTo() +":"+ msg.getToPort();
      if(msg.isRequest()){
         return msg.getTo()+":"+msg.getToPort();
      }else{
         return msg.getFirstViaHost()+":"+msg.getFirstViaPort();
      }
   }
}
