/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package com.voztele.sipspy.flowpanel;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;

public class StraightArrow extends Arrow {
	
	public Dimension dimensionInfo;


	public StraightArrow(boolean selected,String arrowName,int xmin,int xmax,int ymin,int ymax,boolean flag) {
		super(selected, arrowName, flag, xmin, xmax, ymin, ymax);
	}

	public int xmin() {
		return Math.min(xmin, xmax);
	}

	public int xmax() {
		return Math.max(xmin, xmax);
	}

	public int ymin() {
		return Math.min(ymin, ymax);
	}

	public int ymax() {
		return Math.max(ymin, ymax);
	}

	public void draw(Graphics g) {
		String time = sipMessage.getTime();
		time = time.substring(0,time.length()-3);
		String fromPort = sipMessage.getFromPort();
		String toPort = sipMessage.getToPort();
		
		Color c = selected?Color.red:color;
		g.setColor(c);
		Font font = g.getFont();
		font = new Font(font.getName(), Font.BOLD | Font.ITALIC, 12);
		Font smallFont = new Font(font.getName(), Font.BOLD | Font.ITALIC, 8);
		
		g.setFont(font);

		int y = (ymin + ymax) / 2;
		g.drawLine(Math.min(xmin, xmax),y - 1,Math.max(xmin, xmax),y - 1);
		g.drawLine(Math.min(xmin, xmax),y    ,Math.max(xmin,xmax) ,y);
		g.drawLine(Math.min(xmin, xmax),y + 1,Math.max(xmax, xmax),y + 1);
		
		String timeString;
		gc.setTimeInMillis(Long.parseLong(time));
		timeString = sdf.format(gc.getTime());
		
		int timeStringWidth =g.getFontMetrics(g.getFont()).stringWidth(timeString);
		int fistLineStringWidth =g.getFontMetrics(g.getFont()).stringWidth(sipMessage.getFirstLine());
		int smallFontHeight = g.getFontMetrics(smallFont).getHeight();
		int fromPortWidth = g.getFontMetrics(smallFont).stringWidth(fromPort);
		//Flecha de izquierda a derecha
		
		if (xmax > xmin) {
			g.setColor(Color.BLACK);
			g.setFont(smallFont);
			g.drawString(fromPort,xmin-fromPortWidth-1,y+smallFontHeight/2);
			g.drawString(toPort,xmax,y+smallFontHeight/2);
			g.setFont(font);
			g.setColor(c);
			
			g.drawString(sipMessage.getFirstLine(),xmin+StaticTracesCanvas.HORIZONTAL_GAP/2-fistLineStringWidth/2,y-5);
			g.drawString(timeString,xmin+StaticTracesCanvas.HORIZONTAL_GAP/2-timeStringWidth/2,y+g.getFontMetrics(g.getFont()).getHeight());
			//La parte de arriba de la flechita
			g.drawLine(xmax  ,y,xmax-10,y-5);
			g.drawLine(xmax-1,y,xmax-11,y-5);
			g.drawLine(xmax-2,y,xmax-12,y-5);
			//la parte de abajo de la flechita.
			//Para que quede mas gorda, ponemos 3 lineas
			g.drawLine(xmax  ,y,xmax-10,y+5);
			g.drawLine(xmax-1,y,xmax-11,y+5);
			g.drawLine(xmax-2,y,xmax-12,y+5);
		} else {
			g.setColor(Color.BLACK);
			g.setFont(smallFont);
			g.drawString(fromPort,xmax-fromPortWidth-1,y+smallFontHeight/2);
			g.drawString(toPort,xmin,y+smallFontHeight/2);
			g.setFont(font);
			g.setColor(c);
			
			g.drawString(sipMessage.getFirstLine(),xmax+ StaticTracesCanvas.HORIZONTAL_GAP/2-fistLineStringWidth/2,y-2);
			g.drawString(timeString,xmax+StaticTracesCanvas.HORIZONTAL_GAP/2-timeStringWidth/2,y+2+g.getFontMetrics(g.getFont()).getHeight());

			g.drawLine(xmax, y, xmax + 10, y + 5);
			g.drawLine(xmax + 1, y, xmax + 11, y + 5);
			g.drawLine(xmax + 2, y, xmax + 12, y + 5);
			
			g.drawLine(xmax, y, xmax + 10, y - 5);
			g.drawLine(xmax + 1, y, xmax + 11, y - 5);
			g.drawLine(xmax + 2, y, xmax + 12, y - 5);
		} // else
	}
}
