/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <arpa/inet.h>
#include <sys/poll.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "md5.h"
#include "md5global.h"
#include "spyAgent.h"

#define SECRET "lettherebeSipSpy"
#define SECRET_LEN 16
#define RAND_SECRET_LEN 32
#define MAX_AUTH_TIME 10


static char fourbits2char[16] = { '0', '1', '2', '3', '4', '5',
           '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };

char sec_rand[RAND_SECRET_LEN + 1];

void generate_random_secret(void)
{
   int i;
   srandom(time(0));
   for(i = 0; i < RAND_SECRET_LEN; i++) {
      sec_rand[i] = 32 + (int)(95.0 * rand() / (RAND_MAX + 1.0));
   }
   sec_rand[i]=0;
   log(DEBUG,"secret=%s\n",sec_rand);
}

/*
 * Convert an integer to its hex representation,
 * destination array must be at least 8 bytes long,
 * this string is NOT zero terminated
 */
static inline void integer2hex(char* _d, int _s)
{
   int i;
   unsigned char j;
   char* s;

   _s = htonl(_s);
   s = (char*)&_s;

   for (i = 0; i < 4; i++) {

      j = (s[i] >> 4) & 0xf;
      if (j <= 9) {
	 _d[i * 2] = (j + '0');
      } else {
	 _d[i * 2] = (j + 'a' - 10);
      }

      j = s[i] & 0xf;
      if (j <= 9) {
	 _d[i * 2 + 1] = (j + '0');
      } else {
	 _d[i * 2 + 1] = (j + 'a' - 10);
      }
   }
}


inline int string2hex(
        /* input */ unsigned char *str, int len,
        /* output */ char *hex )
{
   int orig_len;

   if (len==0) {
      *hex='0';
      return 1;
   }

   orig_len=len;
   while ( len ) {
      *hex=fourbits2char[(*str) >> 4];
      hex++;
      *hex=fourbits2char[(*str) & 0xf];
      hex++;
      len--;
      str++;
   }
   return orig_len-len;
}

/*
 * Calculate nonce value
 * Nonce value consists of time in seconds since 1.1 1970 and
 * secret phrase
 */
void calc_nonce(char* _nonce, int _expires)
{
   MD5_CTX ctx;
   unsigned char bin[16];

   MD5Init(&ctx);

   integer2hex(_nonce, _expires);
   MD5Update(&ctx, _nonce, 8);

   MD5Update(&ctx, SECRET, SECRET_LEN);
   MD5Final(bin, &ctx);
   string2hex(bin, 16, _nonce + 8);
   _nonce[8 + 32] = '\0';
}
