/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <pcap.h>
#include <net/ethernet.h>
#include <sys/poll.h>
#include <string.h>
#include <sys/socket.h>/*inet_ntoa*/
#include <netinet/in.h>/*inet_ntoa*/
#include <arpa/inet.h>/*inet_ntoa,ntohs*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "spyAgent.h"
#include "capture.h"

#if USE_PCRE
#include "../pcre-5.0/pcre.h"
#else
#include "../regex-0.12/regex.h"
#endif

#if defined(BSD) || defined(SOLARIS) || defined(MACOSX)
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <net/if.h>
#include <sys/tty.h>
#include <pwd.h>
#endif

#if defined(OSF1)
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <net/route.h>
#include <sys/mbuf.h>
#include <arpa/inet.h>
#include <unistd>
#include <pwd.h>
#endif

#if defined(LINUX)
#include <getopt.h>
#include <arpa/inet.h>
#include <ctype.h>
#include <time.h>
#include <unistd.h>
#include <pwd.h>
#include <grp.h>
#endif

#if defined(AIX)
#include <sys/machine.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <time.h>
#include <unistd.h>
#include <pwd.h>
#endif

#if defined(_WIN32)
#include <io.h>
#include <getopt.h>
#include <winsock2.h>
#include <types.h>
#include <inet_ntop.h>

#define strcasecmp stricmp

#else

#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>
#include <netinet/igmp.h>

#endif


extern int newsockfd;
extern int sessionCount;
extern struct sessionS *sessions;

int datalink;
int layer2offset;

#define NOT_MY_PORT "not src port %5d"
#define NOT_MY_PORT_LEN 18


/**This function is called from the main thread, when the PCAP FileDescriptor 
 * has something to read.
 * we first get the packet from libpcap.
 * then we get reception time, from and to IP and Port, and a pointer to the content (cookRawPacket)
 * then we check if it is SIP message, (isSip)
 * then we do a little bit of parsing of the SIP message and extract the transactionId, callId and FirstLine (cookSipPacket)
 */
int capture_packet(const u_char *packet,const struct pcap_pkthdr *header,packetT *cookedPacket)
{
   if ((packet != NULL) && cookRawPacket(header, packet, cookedPacket) && isSip(cookedPacket))
      return cookSipPacket(cookedPacket);
   else 
      log(DEBUG,"Not sip Packet, maybe next time !\n");
   return 0;
}

int isSip(packetT *cookedPacket)
{
   char* car;
   char* message;
   int len;
   len=cookedPacket->message.len;
   message=cookedPacket->message.s;
   car=message;
   while(car!=message+len && (*car!='S' || memcmp(car,"SIP/2.0",7))) car++;
   if(car==message+len)return 0;
   else return 1;
}
#define eat_lws(p,end) while((p)<(end) && (*(p)==' ' || *(p)=='\t')){p++;} 

int cookSipPacket(packetT *p)
{
   char *car,*message,*end;
   int len;
   message = p->message.s;
   len = p->message.len;

   // note that in ANSI C, the second part of a && operator is only evaluated if the first returned 1, and in
   // || operator, the second part is only evaluated if the first returned 0.
   car=message;
   if(!memcmp(message,"SIP/2.0",7))
      p->type=SIP_RESPONSE;
   else
      p->type=SIP_REQUEST;
   while(car<message+len && (*car!='C' || strncasecmp(car,"Call-ID",7))) car++;
   if(car==message+len){
      log(ERROR,"no callid\n");
      goto error;
   }else{
      car+=7;
      eat_lws(car,message+len);
      if (*car!=':') {
 log(ERROR,"bad callid\n");
 goto error;
      }else
 car++;
      eat_lws(car,message+len);
      end=car+1;
      while(end!=message+len && *end!=';' && *end != '\r')end++;
      p->callId.s=car;
      p->callId.len=end-car;
   }
   if(car==message+len)
      return 0;
   car=message;
   while(car!=message+len && (*car!='V' || strncasecmp(car,"Via",3))) car++;
   if(car==message+len){
      goto error;
   }else{
      car+=3;
      eat_lws(car,message+len);
      if(*car!=':' || car==(message+len))
 goto error;
      car++;
      eat_lws(car,message+len);
      /*now car should point to the via1 SIP/2.0/XXX */
      car+=11;
      eat_lws(car,message+len);
      /*now car should point to the via1 hostname start*/
      for(end=car;end<(message+len) && *end!=':' && *end!=' ' && *end!=',' && *end!=';' && *end!='\r' && *end!='\n';end++);
      /*now end should point to the via1 hostname end*/
      if(end==message+len){
 strcat(p->errors.s,"Unable to find Via host:port\n");
 goto error;
      }
      p->firstViaHost.s=car;
      p->firstViaHost.len=end-car;
      if(*end==':'){
 car=end+1;
 for(end=car;end<(message+len) && *end!=' ' && *end!=',' && *end!=';' && *end!='\r' && *end!='\n';end++);
 if(end==message+len){
    strcat(p->errors.s,"Unable to find Via port\n");
    goto error;
 }
 p->firstViaPort.s=car;
 p->firstViaPort.len=end-car;
      }else{
 p->firstViaPort.s="5060";
 p->firstViaPort.len=4;
      }
      car=end;
      while(car<message+len && (*car!=';' || strncasecmp(car+1,"branch=",7)))car++;//looking for the first branch parameter...
      if(car==message+len)
 goto error;
      car+=8;
      end=car;
      while(end<message+len && *end!=';' && *end!=',' && *end!='\r' && *end!=' ')end++;
      if(end==message+len)
	 goto error;
      p->transactionId.s=car;
      p->transactionId.len=end-car;
   }
   car=message;
   while(car<message+len && *car!='\r') car++;
   if(car==message+len)
      return 0;
   p->firstLine.s=message;
   p->firstLine.len=car-message;
   log(DEBUG,"Nice packet:\n \
	 isRequest:\t%s\n \
	 from:\t%s\n \
	 to:\t%s\n \
	 fromPort:\t%hu\n \
	 toPort:\t%hu\n \
	 microseconds:\t%ld%ld\n \
	 transactionID:\t%.*s\n \
	 Call-ID:\t%.*s\n \
	 First Line:\t%.*s\n \
	 First Via:\t%.*s:%.*s\n \
	 Message:\n%.*s\n",p->type==SIP_REQUEST?"yes":"no",p->from,\
 p->to,p->fromPort,p->toPort,\
 p->seconds,p->microseconds,p->transactionId.len,p->transactionId.s,\
 p->callId.len,p->callId.s,p->firstLine.len,p->firstLine.s,\
 p->firstViaHost.len,p->firstViaHost.s,\
 p->firstViaPort.len,p->firstViaPort.s,\
 p->message.len,p->message.s);
   return 1;
error:
   return 0;
}

int packetMatches(sessionT *session,packetT *packet)
{
   if(session==NULL || !(session->regexp.s) )
      return 0;
   if(session->src_ip.s_addr && memcmp(&packet->from,&session->src_ip,sizeof(struct in_addr)))
      return 0;
   if(session->dst_ip.s_addr && memcmp(&packet->to,&session->dst_ip,sizeof(struct in_addr)))
      return 0;
#if USE_PCRE
   switch(pcre_exec(session->pattern,session->pattern_extra,packet->message.s, (int32_t)packet->message.len, 0, 0, 0, 0)) {
      case PCRE_ERROR_NULL:
      case PCRE_ERROR_BADOPTION:
      case PCRE_ERROR_BADMAGIC:
      case PCRE_ERROR_UNKNOWN_NODE:
      case PCRE_ERROR_NOMEMORY:
 perror("she's dead, jim\n");
      case PCRE_ERROR_NOMATCH:
 return 0;
   }
#else
   switch (re_match(session->pattern,packet->message.s, (int32_t)packet->message.len, 0, 0)) {
      case -2:
 perror("she's dead, jim\n");
      case -1:
 return 0;
   }
#endif
   return 1;
}

#define CDATAEND "]]>\n</message>\n\n"
#define CDATAEND_LEN (sizeof(CDATAEND)-1)

int generateXml(packetT *cookedPacket,char *buffer,int len)
{
   int i=0,j=0;
   i=snprintf(buffer,len,"<message\ntype=\"%s\"\nfrom=\"%s\"\nto=\"%s\"\nfromPort=\"%hu\"\n"
 "toPort=\"%hu\"\ntime=\"%ld%06ld\"\nisSender=\"false\"\ntransactionId=\"%.*s\"\ncallId=\"%.*s\"\n"
 "firstLine=\"%.*s\"\nfirstViaHost=\"%.*s\"\nfirstViaPort=\"%.*s\"\ndebugLine=\"0\"\n>\n<![CDATA[", \
 cookedPacket->type==SIP_REQUEST?"1":"2",\
 cookedPacket->from,\
 cookedPacket->to,\
 cookedPacket->fromPort,\
 cookedPacket->toPort,\
 cookedPacket->seconds,\
 cookedPacket->microseconds,\
 cookedPacket->transactionId.len,\
 cookedPacket->transactionId.s,\
 cookedPacket->callId.len,\
 cookedPacket->callId.s,\
 cookedPacket->firstLine.len, \
 cookedPacket->firstLine.s, \
 cookedPacket->firstViaHost.len, \
 cookedPacket->firstViaHost.s, \
 cookedPacket->firstViaPort.len, \
 cookedPacket->firstViaPort.s);
   if(i<=0 || (i>len)){
      log(ERROR,"ERROR writing xml\n");
      return -1;
   }
   memcpy(buffer+i,cookedPacket->message.s,cookedPacket->message.len);
   memcpy(buffer+i+(cookedPacket->message.len),CDATAEND,CDATAEND_LEN);
   for(j=0;j<i+cookedPacket->message.len+CDATAEND_LEN;j++)
      if(buffer[j]==0){
 buffer[j]='.';
      }
   return(CDATAEND_LEN+i+(cookedPacket->message.len));
}

enum sessionErr begin_session(char *dev, char *str_filter, pcap_t **handle)
{
   char errbuf[PCAP_ERRBUF_SIZE];
   char message[MESSAGE_SIZE];
   char *filter2;
   char isfile=0;

   struct bpf_program filter;
   filter2=NULL;

   memset(message,0, MESSAGE_SIZE);
   if (!dev || strlen(dev)==0) {
      dev="any";
   }
   if(!strncmp(dev,"file:",5)){
      isfile=1;
      log(DEBUG,"Opening file \"%s\" for capture, with filter \"%s\"...\n",dev+5,str_filter==NULL?"NULL":str_filter);
      *handle = pcap_open_offline(dev+5, errbuf);
   }else{
      log(DEBUG,"Opening device \"%s\" for capture, with filter \"%s\"...\n",dev==NULL?"NULL":dev,str_filter==NULL?"NULL":str_filter);
      *handle = pcap_open_live (dev, BUFSIZ, 1, 1, errbuf);
   }

   if (*handle == NULL) {
      log(ERROR,"Error in pcap_open_%s opening %s : %s\n",isfile?"offline":"live",dev==NULL?"NULL":dev, errbuf);
      log(ERROR,"Must be root?\n");
      return ERR_OPEN;
   }

   if(str_filter==NULL)
      str_filter="port 5060";
   if (!(filter2=malloc(strlen(str_filter)+40))) {
      log(ERROR,"Out of memory!\n");
   }else{
      if(0>sprintf(filter2,"not src port %d and (%s)",portno,str_filter)){
 log(ERROR,"error sprintf'ing\n");
 free(filter2);
 filter2=NULL;
      }else{
 log(DEBUG,"Setting filter: [%s]\n",filter2);
      }
   }
   if (pcap_compile(*handle, &filter,filter2?filter2:str_filter,0,0)<0) {
      log(ERROR,"Error (%s) in pcap_compile\n",pcap_geterr(*handle));
      pcap_close(*handle);
      *handle=NULL;
      return ERR_FILTER;
   }
   if(filter2){
      free(filter2);
      filter2=NULL;
   }
   if (pcap_setfilter(*handle, &filter)<0) {
      log(ERROR,"Error (%s) in pcap_setfilter\n",pcap_geterr(*handle));
      pcap_close(*handle);
      pcap_freecode(&filter);
      *handle=NULL;
      return ERR_FILTER;
   }
   pcap_freecode(&filter);
   if(pcap_setnonblock(*handle, 1,errbuf)<0){
      printf("Unable to set nonblocking mode to pcap_handle:\n");
      printf("%s\n",errbuf);
   }

   datalink = pcap_datalink(*handle);
   switch (datalink) {
        case DLT_EN10MB:
            layer2offset = ETHHDR_SIZE;
            break;
        case DLT_IEEE802:
            layer2offset = TOKENRING_SIZE;
            break;
        case DLT_FDDI:
            layer2offset = FDDIHDR_SIZE;
            break;
        case DLT_SLIP:
            layer2offset = SLIPHDR_SIZE;
            break;
        case DLT_PPP:
            layer2offset = PPPHDR_SIZE;
            break;
#if HAVE_DLT_LOOP
        case DLT_LOOP:
#endif
        case DLT_NULL:
            layer2offset = LOOPHDR_SIZE;
            break;
#if HAVE_DLT_RAW
        case DLT_RAW:
            layer2offset = RAWHDR_SIZE;
            break;
#endif
#if HAVE_DLT_LINUX_SLL
        case DLT_LINUX_SLL:
            layer2offset = ISDNHDR_SIZE;
            break;
#endif
#if HAVE_DLT_IEEE802_11
        case DLT_IEEE802_11:
            layer2offset = IEEE80211HDR_SIZE;
            break;
#endif
        default:
            fprintf(stderr, "fatal: unsupported interface type %u\n", datalink);
            exit(-1);
   }
   return SUCCESS;
}

int cookRawPacket(const struct pcap_pkthdr *h, const u_char *p,packetT *paquet)
{
   /*void process(u_char *d, struct pcap_pkthdr *h, u_char *p) {*/
   struct ip      *ip_packet  = (struct ip *)    (p + layer2offset);
#if USE_IPv6
   struct ip6_hdr *ip6_packet = (struct ip6_hdr*)(p + layer2offset);
#endif

   uint32_t ip_ver   = ip_packet->ip_v;

   uint8_t  ip_proto = 0;
   uint32_t ip_hl    = 0;
   uint32_t ip_off   = 0;

   uint8_t  fragmented  = 0;
   uint16_t frag_offset = 0;
   uint32_t frag_id     = 0;

   char ip_src[INET6_ADDRSTRLEN + 1],
	ip_dst[INET6_ADDRSTRLEN + 1];

   unsigned char *data;
   uint32_t len = 0;
   
   //in host format, which in 80X86 is Least Significant Byte First (network is MSBF)
   paquet->seconds = h->ts.tv_sec;
   paquet->microseconds = h->ts.tv_usec;

   switch (ip_ver) {
      case 4: {
#if defined(AIX)
#undef ip_hl
		 ip_hl       = ip_packet->ip_ff.ip_fhl * 4;
#else
		 ip_hl       = ip_packet->ip_hl * 4;
#endif
		 ip_proto    = ip_packet->ip_p;
		 ip_off      = ntohs(ip_packet->ip_off);
		 fragmented  = ip_off & (IP_MF | IP_OFFMASK);
		 frag_offset = (fragmented) ? (ip_off & IP_OFFMASK) * 8 : 0;
		 frag_id     = ntohs(ip_packet->ip_id);
		 inet_ntop(AF_INET, (const void *)&ip_packet->ip_src,paquet->from, sizeof(ip_src));
		 inet_ntop(AF_INET, (const void *)&ip_packet->ip_dst,paquet->to, sizeof(ip_dst));
	      } break;
#if USE_IPv6
      case 6: {
		 ip_hl    = sizeof(struct ip6_hdr);
		 ip_proto = ip6_packet->ip6_nxt;
		 if (ip_proto == IPPROTO_FRAGMENT) {
		    struct ip6_frag *ip6_fraghdr;

		    ip6_fraghdr = (struct ip6_frag *)(((unsigned char *)ip6_packet) + ip_hl);
		    ip_hl      += sizeof(struct ip6_frag);
		    ip_proto    = ip6_fraghdr->ip6f_nxt;

		    fragmented  = 1;
		    frag_offset = ntohs(ip6_fraghdr->ip6f_offlg & IP6F_OFF_MASK);
		    frag_id     = ntohl(ip6_fraghdr->ip6f_ident);
		 }

		 inet_ntop(AF_INET6, (const void *)&ip6_packet->ip6_src,paquet->from, sizeof(ip_src));
		 inet_ntop(AF_INET6, (const void *)&ip6_packet->ip6_dst,paquet->to, sizeof(ip_dst));
	      } break;
#endif
   }

   switch (ip_proto) {
      case IPPROTO_TCP: {
			   struct tcphdr *tcp     = (struct tcphdr *)(((unsigned char *)ip_packet) + ip_hl);
			   uint16_t tcphdr_offset = (frag_offset) ? 0 : (tcp->th_off * 4);

			   data = ((unsigned char *)tcp) + tcphdr_offset;

			   switch (ip_ver) {
			      case 4: {
					 if ((len = ntohs(ip_packet->ip_len)) < h->caplen)
					    len -= ip_hl + tcphdr_offset;
					 else
					    len = h->caplen - layer2offset - ip_hl - tcphdr_offset;
				      } break;

#if USE_IPv6
			      case 6: {
					 if ((len = ntohs(ip6_packet->ip6_plen + ip_hl)) < h->caplen)
					    len -= ip_hl + tcphdr_offset;
					 else
					    len = h->caplen - layer2offset - ip_hl - tcphdr_offset;
				      } break;
#endif
			   }
			   paquet->fromPort=ntohs(tcp->th_sport);
			   paquet->toPort=ntohs(tcp->th_dport);
			   paquet->message.len = len;
			   memcpy(paquet->message.s,data,paquet->message.len);
			   paquet->message.s[len] = 0;
			} break;

      case IPPROTO_UDP: {
			   struct udphdr *udp     = (struct udphdr *)(((unsigned char *)ip_packet) + ip_hl);
			   uint16_t udphdr_offset = (frag_offset) ? 0 : sizeof(struct udphdr);

			   data = ((unsigned char *)udp) + udphdr_offset;

			   switch (ip_ver) {
			      case 4: {
					 if ((len = ntohs(ip_packet->ip_len)) < h->caplen)
					    len -= ip_hl + udphdr_offset;
					 else
					    len = h->caplen - layer2offset - ip_hl - udphdr_offset;
				      } break;

#if USE_IPv6
			      case 6: {
					 if ((len = ntohs(ip6_packet->ip6_plen + ip_hl)) < h->caplen)
					    len -= ip_hl + udphdr_offset;
					 else
					    len = h->caplen - layer2offset - ip_hl - udphdr_offset;
				      } break;
#endif
			   }

			   paquet->fromPort=ntohs(udp->uh_sport);
			   paquet->toPort=ntohs(udp->uh_dport);
			   paquet->message.len = len;
			   memcpy(paquet->message.s,data,paquet->message.len);
			   paquet->message.s[len] = 0;
			} break;
      default: 
			return 0;
			break;

   }
   return 1;
}

packetT *alloc_packet_mem(int messageSize)
{

   packetT *yourPacket;
   yourPacket = (packetT *) malloc(sizeof(packetT));
   yourPacket->errors.s=(char *)malloc(500);
   yourPacket->message.s=(char *)malloc(messageSize);
   return yourPacket;
}

void free_packet_mem(packetT *p)
{
   free(p->message.s);
   free(p->errors.s);
   free(p);
}
