/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include <stdlib.h>/*free, malloc*/
#include <string.h>/*strcpy*/
#include <unistd.h>/*write*/
#include <errno.h>/*errno*/
#include <time.h>
#include <sys/types.h>/*open*/
#include <sys/stat.h>/*open*/
#include <fcntl.h>/*open*/
#include <dirent.h>/*readdir*/
#include <string.h>/*strcmp*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#if USE_PCRE
#include "../pcre-5.0/pcre.h"
#else
#include "../regex-0.12/regex.h"
#endif


#include "spyAgent.h"
#include "commands.h"
#include "capture.h"
#include "auth.h"
#include "md5.h"


#define writeMsg(fd, msg) \
   do{\
      write(fd, msg, strlen(msg));\
   }while(0)
char *dirname="sessions";

enum cmdStat device_command(sessionT *session,str arg,int cseq)
{
   int i;
   char superbuf[500];
   char *errmsg;
   errmsg=NULL;
   if (!session->authenticated || !(session->role & ADMIN_ROLE)) {
      errmsg="unautorized to change device";
      log(WARN,"Unautorized attempt to change session Device\n");
      goto error2;
   }
   if(session->dev.s)
      free(session->dev.s);
   session->dev=arg;
   log(DEBUG,"New client's device %.*s\n",arg.len,arg.s);
   if(!secure){
      if(pcap_handle){
	 pcap_close(pcap_handle);
	 pcap_handle=NULL;
      }
      switch(begin_session(session->dev.s,session->filter.s, &pcap_handle)){
	 case SUCCESS:
	    pollFds[0].fd = pcap_get_selectable_fd(pcap_handle);
	    pollFds[0].events = POLLIN|POLLHUP;
	    pollFds[0].revents = 0;
	    i=snprintf(superbuf,500,"<response status=\"ok\" command=\"device\" cseq=\"%d\" />\n",cseq);
	    write(session->controlFd,superbuf,i);
	    break;
	 case ERR_LOOKUPNET:
	    errmsg="(lookupnet error)";
	    goto error;
	 case ERR_FILTER:
	    errmsg="(filter error)";
	    goto error;
	 case ERR_OPEN:
	    errmsg="(error opening pcap_handle)";
	    goto error;
	 default:
	    errmsg="(unknown error)";
	    goto error;
      }
   }else{
      i=snprintf(superbuf,500,"<response status=\"locked\" command=\"device\" cseq=\"%d\"/>\n",cseq);
      write(session->controlFd,superbuf,i);
   }
   return CS_SUCCESS;
error:
   pcap_handle=NULL;
   if(session->dev.s){
      free(session->dev.s);
      session->dev.s=NULL;
      session->dev.len=0;
      arg.s=NULL;
   }
error2:
   if(arg.s)
      free(arg.s);
   log(WARN,"Error device: %s\n",errmsg);
   i=snprintf(superbuf,500,"<response status=\"error\" command=\"device\" reason=\"%s\" cseq=\"%d\" />\n",\
	 errmsg,cseq);
   write(session->controlFd,superbuf,i);
   return CS_ERROR;
}

enum cmdStat username_command(sessionT *session,str arg,int cseq)
{
   int i;
   char superbuf[500];
   char *errmsg;
   errmsg=NULL;
   log(DEBUG,"New client's username %.*s\n",arg.len,arg.s);
   if(session->authenticated){
      errmsg="Already authenticated.Log out first.";
      log(ERROR,"already authenticated user\n");
      goto error;
   }
   for (i=0;i<usersNum;i++)
      if (!memcmp(userlist[i].username,arg.s,arg.len))
	 break;
   if(i==usersNum){
      errmsg="Inexistant user name";
      goto error;
   }
   session->regex_pattern=userlist[i].pattern;
#if USE_PCRE
   session->regex_pattern_extra=userlist[i].pattern_extra;
#endif
   session->username=arg;
   session->password.s=userlist[i].password;
   session->password.len=strlen(userlist[i].password);
   if (!strcmp(userlist[i].role,ADMIN_ROLE_S)) {
      session->role|=ADMIN_ROLE;
   }else if (!strcmp(userlist[i].role,USER_ROLE_S)) {
      session->role|=USER_ROLE;
   }else{
      log(ERROR,"Unknown role type: %.*s\n",userlist[i].role);
      session->role=0;
   }
   if(session->nonce.s)
      free(session->nonce.s);
   if (!(session->nonce.s=malloc(41))) {
      log(ERROR,"Unable to malloc for username %.*s\n",arg.len,arg.s);
      errmsg="(internal error)";
      goto error;
   }
   calc_nonce(session->nonce.s,time(0));
   session->nonce.s[40]=0;
   session->nonce.len=40;
   log(DEBUG,"created nonce:%.*s\n",session->nonce.len,session->nonce.s);
   i=snprintf(superbuf,500,"<response status=\"ok\" nonce=\"%.*s\" command=\"username\" cseq=\"%d\" />",\
	 session->nonce.len,session->nonce.s,cseq);
   write(session->controlFd,superbuf,i);
   return CS_SUCCESS;
error:
   log(WARN,"Error username: %s\n",errmsg);
   i=snprintf(superbuf,500,"<response status=\"error\" command=\"username\" reason=\"%s\" cseq=\"%d\" />",\
	 errmsg,cseq);
   write(session->controlFd,superbuf,i);
   return CS_ERROR;
}

enum cmdStat password_command(sessionT *session,str arg,int cseq)
{
   int i;
   char superbuf[500];
   char *errmsg,password[33];
   unsigned char bin[16];
   MD5_CTX ctx;

   errmsg=NULL;
   log(DEBUG,"New client's password %.*s\n",arg.len,arg.s);
   if (!session->username.s) {
      errmsg="send username first!";
      goto error;
   }
   MD5Init(&ctx);
   MD5Update(&ctx,session->nonce.s,session->nonce.len);
   MD5Update(&ctx,session->password.s,session->password.len);
   MD5Final(bin, &ctx);
   string2hex(bin, 16,password);
   password[32]=0;
   if (strncmp(arg.s,password,arg.len)) {
      errmsg="Wrong password";
      log(DEBUG,"received password:[%.*s] real password:[%s]\n",arg.len,arg.s,password);
      goto error;
   }
   i=snprintf(superbuf,500,"<response status=\"ok\" command=\"password\" cseq=\"%d\" />",cseq);
   write(session->controlFd,superbuf,i);
   session->authenticated=TRUE;
   if(session->role & ADMIN_ROLE)
      writeMsg(session->controlFd,devices);
   free(arg.s);
   return CS_SUCCESS;
error:
   log(WARN,"Error username: %s\n",errmsg);
   i=snprintf(superbuf,500,"<response status=\"error\" command=\"password\" reason=\"%s\" cseq=\"%d\" />",\
	 errmsg,cseq);
   write(session->controlFd,superbuf,i);
   free(arg.s);
   return CS_ERROR;
}

enum cmdStat filter_command(sessionT *session,str arg,int cseq)
{
   int i;
   char superbuf[500];
   char *errmsg;
   struct bpf_program filter;
   errmsg=NULL;
   if(!session->authenticated || !(session->role & ADMIN_ROLE)) {
      errmsg="unautorized to change filter";
      log(WARN,"Unautorized attempt to change session Filter\n");
      goto error2;
   }
   if(session->filter.s)
      free(session->filter.s);
   session->filter=arg;
   log(DEBUG,"New client's filter %.*s\n",arg.len,arg.s);
   if(pcap_handle){
      if (pcap_compile(pcap_handle, &filter,session->filter.s,0,0)<0) {
	 log(ERROR,"Error (%s) in pcap_compile\n",pcap_geterr(pcap_handle));
	 errmsg="(unable to compile filter)";
	 goto error;
      }
      if (pcap_setfilter(pcap_handle, &filter)<0) {
	 log(ERROR,"Error (%s) in pcap_setfilter\n",pcap_geterr(pcap_handle));
	 errmsg="(unable to set filter)";
	 goto error;
      }
      pcap_freecode(&filter);
   }
   i=snprintf(superbuf,500,"<response status=\"ok\" command=\"filter\" cseq=\"%d\"/>\n",cseq);
   write(session->controlFd,superbuf,i);
   return CS_SUCCESS;
error:
   log(WARN,"Error filter: %s\n",errmsg);
   if(session->filter.s){
      free(session->filter.s);
      session->filter.s=NULL;
      session->filter.len=0;
      arg.s=NULL;
   }
error2:
   if(arg.s)
      free(arg.s);
   i=snprintf(superbuf,500,"<response command=\"filter\" status=\"error\" reason=\"%s\" cseq=\"%d\" />\n",\
	 errmsg,cseq);
   write(session->controlFd,superbuf,i);
   return CS_ERROR;
}

enum cmdStat regexp_command(sessionT *session,str arg,int cseq)
{
   int i;
   char superbuf[500];
   const char *re_err;
   const char *errmsg=NULL;
#if USE_PCRE
   int32_t err_offset;
   uint32_t pcre_options = PCRE_UNGREEDY;
#else
   re_syntax_options = RE_CHAR_CLASSES | RE_NO_BK_PARENS | RE_NO_BK_VBAR | RE_CONTEXT_INDEP_ANCHORS | RE_CONTEXT_INDEP_OPS;
#endif
   if (!session->authenticated) {
      errmsg="please, authenticate first\n";
      goto error;
   }
#if USE_PCRE
   switch(pcre_exec(session->regex_pattern, session->regex_pattern_extra, arg.s, arg.len, 0, 0, 0, 0)) {
      case PCRE_ERROR_NULL:
      case PCRE_ERROR_BADOPTION:
      case PCRE_ERROR_BADMAGIC:
      case PCRE_ERROR_UNKNOWN_NODE:
      case PCRE_ERROR_NOMEMORY:
	 errmsg="(internal error in regexp searching)";
	 goto error;
      case PCRE_ERROR_NOMATCH:
	 errmsg="(You're not allowed to use this regex)";
	 goto error;
   }
#else
   switch (re_match(session->regex_pattern,arg.s,arg.len+1,0,0)) {
      case -2:
	 errmsg="(internal error in regexp searching)";
	 goto error;
      case -1:
	 errmsg="(You're not allowed to use this regex)";
	 goto error;
   }
#endif
   session->regexp=arg;
#if USE_PCRE
   if(session->pattern)
      pcre_free(session->pattern);
   session->pattern = pcre_compile(session->regexp.s, pcre_options, (const char **)&re_err, &err_offset, 0);
   if (!session->pattern) {
      log(ERROR, "Compilation Failed: %s\n", re_err);
      errmsg=re_err;
      goto error;
   }
   if(session->pattern_extra)
      pcre_free(session->pattern_extra);
   session->pattern_extra = pcre_study(session->pattern, 0, (const char **)&re_err);
   if(re_err!=NULL){
      log(ERROR,"pcre_study error %s\n",re_err);
      if (session->pattern_extra) {
	 pcre_free(session->pattern_extra);
	 session->pattern_extra=NULL;
      }
   }
#else
   if(session->pattern)
      free(session->pattern);
   if(!(session->pattern =malloc(sizeof(struct re_pattern_buffer)))){
      log(ERROR,"Out of memory!!\n");
      errmsg="internal error";
      goto error;
   }
   memset(session->pattern,0,sizeof(struct re_pattern_buffer));
   if(!(session->pattern->fastmap =malloc(MAX_PATTERN_LEN))){
      log(ERROR,"Out of memory!!\n");
      errmsg="internal error";
      goto error;
   }
   re_err = re_compile_pattern(session->regexp.s,session->regexp.len,session->pattern);
   if (re_err) {
      log(ERROR, "regex compile: %s\n", re_err);
      errmsg=re_err;
      goto error;
   }
   if (re_compile_fastmap(session->pattern)) {
      log(ERROR,"fastmap compile failed");
      errmsg="internal error";
      goto error;
   }
#endif
   i=snprintf(superbuf,500,"<response status=\"ok\" command=\"regexp\" cseq=\"%d\" />\n",cseq);
   write(session->controlFd,superbuf,i);
   return CS_SUCCESS;
error:
   log(WARN,"Error regexp: %.*s\n",arg.len,arg.s);
   if(session->regexp.s){
      free(session->regexp.s);
      session->regexp.s=NULL;
      session->regexp.len=0;
      arg.s=NULL;
   }
#if USE_PCRE
   if(session->pattern)
      pcre_free(session->pattern);
   session->pattern=NULL;
   if(session->pattern_extra)
      pcre_free(session->pattern_extra);
   session->pattern_extra=NULL;
#else
   if(session->pattern){
      if(session->pattern->fastmap)
	 free(session->pattern->fastmap);
      free(session->pattern);
      session->pattern=NULL;
   }
#endif
   if(arg.s)
      free(arg.s);
   i=snprintf(superbuf,500,"<response status=\"error\" reason=\"%s\" command=\"regexp\" cseq=\"%d\" />\n",\
	 errmsg,cseq);
   write(session->controlFd,superbuf,i);
   return CS_ERROR;
}

enum cmdStat bye_command(sessionT *session,str arg,int cseq)
{
   int i;
   char superbuf[500];
   i=snprintf(superbuf,500,"<response status=\"ok\" command=\"bye\" cseq=\"%d\" />\n",cseq);
   write(session->controlFd,superbuf,i);
   return CS_REMOVE;
}

enum cmdStat src_ip_command(sessionT *session,str arg,int cseq)
{
   int i;
   char superbuf[500];
   char *errmsg;
   if (!session->authenticated) {
      errmsg="please, authenticate first\n";
      goto error;
   }
   inet_aton(arg.s,&session->src_ip);
   free(arg.s);
   i=snprintf(superbuf,500,"<response status=\"ok\" command=\"src_ip\" cseq=\"%d\" />\n",cseq);
   write(session->controlFd,superbuf,i);
   return SUCCESS;
error:
   log(WARN,"Error src_ip: %.*s\n",arg.len,arg.s);
   memset(&session->src_ip,sizeof(struct in_addr),0);
   free(arg.s);
   i=snprintf(superbuf,500,"<response status=\"error\" reason=\"%s\" cseq=\"%d\" command=\"src_ip\" />\n",
	 errmsg,cseq);
   write(session->controlFd,superbuf,i);
   return CS_ERROR;
}

enum cmdStat dst_ip_command(sessionT *session,str arg,int cseq)
{
   int i;
   char superbuf[500];
   char *errmsg;
   if (!session->authenticated) {
      errmsg="please, authenticate first\n";
      goto error;
   }
   inet_aton(arg.s,&session->dst_ip);
   free(arg.s);
   writeMsg(session->controlFd,"<response status=\"ok\" command=\"dst_ip\" />\n");
   return SUCCESS;
error:
   log(WARN,"Error dst_ip: %.*s\n",arg.len,arg.s);
   memset(&session->dst_ip,sizeof(struct in_addr),0);
   free(arg.s);
   i=snprintf(superbuf,500,"<response status=\"error\" reason=\"%s\" cseq=\"%d\" command=\"dst_ip\" />\n",
	 errmsg,cseq);
   write(session->controlFd,superbuf,i);
   return CS_ERROR;
}

enum cmdStat replay_command(sessionT *session,str arg,int cseq)
{
   int i,j,fd,towrite;
   char superbuf[500];
   char dname[100];
   char *errmsg,*basename;
   fd=-1;
   if (!session->authenticated) {
      errmsg="please, authenticate first\n";
      goto error;
   }
   basename=strrchr(arg.s,'/');
   strncpy(dname,dirname,99);
   if(basename){
      basename++;
   }else{
      basename=arg.s;
   }
   strncat(dname,"/",99);
   strncat(dname,basename,99);

   if(0>(fd=open(dname,O_RDONLY))){
      errmsg="Unable go open file\n";
      goto error;
   }else{
      i=snprintf(superbuf,500,"<response status=\"ok\" cseq=\"%d\" command=\"replay\" />\n",
	    errmsg,cseq);
      write(session->controlFd,superbuf,i);
      do{
	 if(0>(i=read(fd,superbuf,500))){
	    errmsg="Unable go open file\n";
	    goto error;
	 }else if(i>0){
	    j=0;
	    towrite=i;
	    while(towrite){
	       if(0>(j=write(session->controlFd,superbuf+(i-towrite),towrite))){
		  errmsg="error writing\n";
		  goto error;
	       }else{
		  towrite-=j;
	       }
	    }
	 }
      }while(i!=0);
   }
   if(fd!=-1)
      close(fd);
   free(arg.s);
   return SUCCESS;
error:
   if(fd!=-1)
      close(fd);
   free(arg.s);
   log(WARN,"Error replay: %.*s\n",arg.len,arg.s);
   i=snprintf(superbuf,500,"<response status=\"error\" reason=\"%s\" cseq=\"%d\" command=\"replay\" />\n",
	 errmsg,cseq);
   write(session->controlFd,superbuf,i);
   return CS_ERROR;
}

enum cmdStat list_command(sessionT *session,str arg,int cseq)
{
   int i,j,fd,towrite;
   char superbuf[500];
   char *errmsg;
   struct stat mistat;
   struct dirent *midir;
   DIR *dp;
   dp=NULL;
   fd=-1;
   if (!session->authenticated) {
      errmsg="please, authenticate first\n";
      goto error;
   }
   if((0>stat(dirname,&mistat)) || !S_ISDIR(mistat.st_mode)){
      errmsg="no dir\n";
      goto error;
   }
   if(NULL==(dp=opendir(dirname))){
      errmsg="cant open\n";
      goto error;
   }
   i=0;
   j=0;
   if(0>(i=snprintf(superbuf,500,"<response status=\"ok\" command=\"list\" cseq=\"%d\" >\n",cseq)) || i>500){
      errmsg="unspec\n";
      goto error;
   }
   while(NULL!=(midir=readdir(dp))){
      if(!strcmp(".",midir->d_name) || !strcmp("..",midir->d_name))
	 continue;
      if(0>(j=snprintf(superbuf+i,500-i,"<file name=\"%s\"/>",midir->d_name)) || j>(500-i)){
	 errmsg="unspec";
	 goto error;
      }else
	 i+=j;
   }
   if(0>(j=snprintf(superbuf+i,500-i,"</response>\n")) || j>(500-i)){
      errmsg="unspec\n";
      goto error;
   }else
      j+=i;
   write(session->controlFd,superbuf,j);
   free(arg.s);
   closedir(dp);
   return SUCCESS;
error:
   if(dp!=NULL)
      closedir(dp);
   free(arg.s);
   log(WARN,"Error list: %.*s\n",arg.len,arg.s);
   i=snprintf(superbuf,500,"<response status=\"error\" reason=\"%s\" cseq=\"%d\" command=\"list\" />\n",
	 errmsg,cseq);
   write(session->controlFd,superbuf,i);
   return CS_ERROR;
}

enum cmdStat save_command(sessionT *session,str arg,int cseq)
{
   int i,fd;
   unsigned int len;
   char superbuf[500];
   char dname[100];
   char *errmsg,*basename;
   str filelength;

   errmsg=NULL;
   if (!session->authenticated) {
      errmsg="unautorized to save file";
      log(WARN,"Unautorized attempt to save file\n");
      goto error;
   }
   filelength=arg;
   while (filelength.len && *filelength.s!=':'){
      filelength.len--;
      filelength.s++;
   }
   if((!filelength.len) || (filelength.len==1)){
      log(ERROR,"Wrong filelength:length\n");
      errmsg="Wrong filelength:length";
      goto error;
   }else{
      *filelength.s=0;
      filelength.len--;
      filelength.s++;
   }
   if(0>str2int(&filelength,&len)){
      log(ERROR,"Wrong file length: %.*s\n",filelength.len,filelength.s);
      errmsg="Wrong file length";
      goto error;
   }
   basename=strrchr(arg.s,'/');
   strncpy(dname,dirname,99);
   if(basename){
      basename++;
   }else{
      basename=arg.s;
   }
   strncat(dname,"/",99);
   strncat(dname,basename,99);
   if(0>(fd=open(dname,O_WRONLY|O_CREAT|O_APPEND , S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP))){
      log(ERROR,"ERROR opening file %s (%s)\n",basename,strerror(errno));
      errmsg="Unable to open file to write\n";
      goto error;
   }
   session->file.total_len=len;
   session->file.read_sofar=0;
   session->reading_file=TRUE;
   session->file.fd=fd;
   i=snprintf(superbuf,500,"<response status=\"ok\" command=\"src_ip\" cseq=\"%d\" />\n",cseq);
   write(session->controlFd,superbuf,i);
   return CS_FILE;
error:
   log(WARN,"Error file: %s\n",errmsg);
   i=snprintf(superbuf,500,"<response status=\"error\" command=\"save\" reason=\"%s\" cseq=\"%d\" />\n",\
	 errmsg,cseq);
   write(session->controlFd,superbuf,i);
   return CS_ERROR;
}

int str2int(str* _s, unsigned int* _r)
{
        int i;

        *_r = 0;
        for(i = 0; i < _s->len; i++) {
                if ((_s->s[i] >= '0') && (_s->s[i] <= '9')) {
                        *_r *= 10;
                        *_r += _s->s[i] - '0';
                } else {
                        return -1;
                }
        }

        return 0;
}


