/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef NETSTUFFH
#define NETSTUFFH
#define IP_RF 0x8000                    /* reserved fragment flag */
#define IP_DF 0x4000                    /* dont fragment flag */
#define IP_MF 0x2000                    /* more fragments flag */
#define IP_OFFMASK 0x1fff               /* mask for fragmenting bits */

//Ethernet header
typedef struct {
	unsigned char dst[6];
	unsigned char src[6];
	unsigned short int proto;
} ethernet_header;

//pcap cooked header
typedef struct {
	unsigned char dummy[14];
	unsigned short int proto;
} cooked_header;

// IPv4 header 
typedef struct {
#if BYTE_ORDER == LITTLE_ENDIAN
	unsigned int ihl:4;
	unsigned int version:4;
#elif BYTE_ORDER == BIG_ENDIAN
	unsigned int version:4;
	unsigned int ihl:4;
#endif
	unsigned char tos;
	unsigned short int length;
	unsigned short int id;
	unsigned short int flags_offset;
	unsigned char ttl;
	unsigned char proto;
	unsigned char h_checksum;
	unsigned long src;
	unsigned long dst;
	//here go options
} ipv4_header;

// IPv6 header 
typedef struct {
	unsigned long version_tclass_flabel;
	unsigned short int length;
	unsigned char next_h;
	unsigned char hop_l;
	unsigned char src[16];
	unsigned char dst[16];
} ipv6_header;

// IPv6 extension header
typedef struct {
	unsigned char next_header;
	unsigned char header_length;
} ipv6_ext_header;

// UDP header 
typedef struct {
	unsigned short int src_port;
	unsigned short int dst_port;
	unsigned short int udp_length;
	unsigned short int udp_checksum;
} udp_header;

// TCP header
typedef struct {
	unsigned short int src_port;
	unsigned short int dst_port;
	unsigned long seq;
	unsigned long ack;
#if BYTE_ORDER == LITTLE_ENDIAN
	u_int8_t unused:4;
	u_int8_t h_length:4;
#elif BYTE_ORDER == BIG_ENDIAN
	u_int8_t h_length:4;
	u_int8_t unused:4;
#endif
	unsigned char flags;
	unsigned short int window;
	unsigned short int tcp_checksum;
	unsigned short int urgent;
	// here go options
} tcp_header;

// ICMPv6 header 
typedef struct {
	unsigned char type;
	unsigned char code;
	unsigned short int checksum;
} icmp6_header;

//headers length 
#define ETH_H   14
#define COOK_H  16
#define IP4_H   20
#define IP6_H   40
#define UDP_H    8
#define TCP_H   20
#define ICMP6_H  4

// Ethernet protocols 
#define IP4 0x0800
#define IP6 0x86DD

// IP protocols 
#ifndef _NETINET_IN_H

#define IPPROTO_IP		 0  
#define IPPROTO_HOP2HOP  0
#define IPPROTO_ICMP     1
#define IPPROTO_IGMP     2
#define IPPROTO_IPIP     4
#define IPPROTO_TCP      6
#define IPPROTO_EGP      8
#define IPPROTO_PUP      12
#define IPPROTO_UDP      17
#define IPPROTO_IDP      22
#define IPPROTO_TP       29
#define IPPROTO_IPV6     41
#define IPPROTO_ROUTING  43
#define IPPROTO_FRAGMENT 44
#define IPPROTO_RSVP     46
#define IPPROTO_GRE      47
#define IPPROTO_ESP      50
#define IPPROTO_AH       51
#define IPPROTO_ICMP6    58
#define IPPROTO_NONE     59
#define IPPROTO_DSTOPTS  60
#define IPPROTO_MTP      92
#define IPPROTO_ENCAP    98
#define IPPROTO_PIM      103
#define IPPROTO_COMP     108
#define IPPROTO_RAW      255

#endif

#endif
