/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

//TODO check socket error conditions
//TODO if the DataChannel is closed at any time and the program tries to send msgs on it, it will die
//TODO it is usual for a client to receive a packet twice: once from the sending host, 
//	and another time from the receiving host (if there was a spyAgent there also).
//	it would be goot to mark a flag in each packet, ie: flag=R or flag=S (for sent or received), so
//	that clients could discern if a packet is beeing seen twice because of that effect. 
//	Calculating an MD5 on the packet would be good also for discerning retransmissions and the
//	effect previously mentioned

#define TCPDUMP_MAGIC 0xa1b2c3d4 //also defined in libpcap/savefile.c

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pcap.h>
#include <net/ethernet.h>
#include <grp.h>
#include <time.h>

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <netinet/tcp.h>/*TCP_NODELAY*/


#include <arpa/inet.h>/*inet_ntoa*/

#include <fcntl.h>
#include <unistd.h>
#include <sys/poll.h>
#include <stdarg.h>
#include <errno.h>
#include <pwd.h>

#include <string.h>
#include <errno.h>
#include <signal.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#if USE_PCRE
#include "../pcre-5.0/pcre.h"
#else
#include "../regex-0.12/regex.h"
#endif

#include "spyAgent.h"
#include "commands.h"
#include "capture.h"
#include "auth.h"


#define DEBUG 0
#define INFO 1
#define WARN 2
#define ERROR 3
#define FATAL 4

#ifndef DEBUG_LEVEL
#define DEBUG_LEVEL DEBUG
#endif

#define FIRSTPORT 32000
#define NUMPORTS 10000
#define BUF_LEN 400


#define MAX_COMMAND 20
#define MAX_LEN_S 5

int debug_level=DEBUG_LEVEL;
int secure=FALSE;
char err,serverIp[256],*devices,*passwordFile;
int usersNum=0;
userT userlist[15];
struct command cmdlist[MAX_COMMAND];
int cmdnr=0;
int portno;
char device[DEVICELEN];
unsigned short int nextPortOffset = 0;
int serverSocketFd,clientId=0;
int signaledSession = -1;
pcap_t *pcap_handle=NULL;
struct pollfd pollFds[MAX_SESSIONS+2];
sessionT **sessions;
int sessionCount=0;
char *xmlMessage;
char *file=0;

packetT *lonePacket;

static inline int calcRegex(userT *usr);

void sigPipe(int signal)
{
   log(ERROR,"SigPipe received ! seems that a session has been closed...\nCleaning up\n");
}

void interrupt(int signal)
{
   int i;
   printf("Exiting ...\n");
   close(serverSocketFd);
   if (pcap_handle) {
      log(FATAL,"Closing pcap session\n");
      pcap_close(pcap_handle);
      pcap_handle=NULL;
      pollFds[0].events =0 ;

   }
   for(i=0;i<sessionCount;i++){
      write(sessions[i]->controlFd,"</messages>\n",12);
      free_session(sessions[i]);
      sessions[i]=NULL;
   }
   free_packet_mem(lonePacket);
   free(xmlMessage);
   free(sessions);
   exit(1);
}

void usage(char **argv)
{
   printf ("Usage: %s [options] <bpf filter>\n"
	 "-h this help\n"
	 "-p <port> listen port for client connections\n"
	 "-l <IP> listen ip for client connections\n"
	 "-S Secure mode: lock the sniffing device and filter, and then drop the root privileges\n"
	 "-d <device name> device where the libpcap must sniff for packets\n"
	 "-f <filename> tcpdump file containing a previous capture(will appear as another device to clients)\n"
	 "-g <number> Debug level (0=DEBUG,1=INFO,2=WARN,3=ERROR,4=FATAL)\n"
	 "-w <filename> filename with name:password\\n entries(to authenticate users)\n"
	 "Recall that packet filter is in libpcap format, it is not the Regular Expression\n"
	 "Default listen port is 23332, (better leave this alone)\n",argv[0]);
   exit(0);	
}

void parseArgs(int argc, char **argv, int *portno, char *listenIp, char **filter,char* device,int devicelen) 
{
   long int aux_port;
   char *port;
   char c;
   port=NULL;
   *filter=NULL;
   while ((c = getopt(argc, argv, "hSp:g:l:d:f:w:")) != EOF) {
      switch (c) {
	 case 'p':
	    port = optarg;
	    break;
	 case 'l':
	    strcpy(listenIp,optarg);
	    break;
	 case 'd':
	    strncpy(device,optarg,devicelen);
	    break;
	 case 'g':
	    debug_level=atoi(optarg);
	    if(debug_level<0 || debug_level>4){
	       printf ("Invalid debug level (%s), setting WARN (2)\n",optarg);
	       debug_level=WARN;
	    }
	    break;
	 case 'h':
	    usage(argv);
	    break;
	 case 'f':
	    file=optarg;
	    break;
	 case 'w':
	    passwordFile=optarg;
	    break;
	 case 'S':
	    secure=TRUE;
	    break;
	 default:
	    usage(argv);
      }
   }
   if(argv[optind]){
      if (!(*filter= get_filter_from_argv(argv+optind))) {
	 log(ERROR,"Unable to create BPF filter from command-line arguments\n");
	 exit(0);
      }
   }
   log(DEBUG,"device: %s\n",device);
   log(DEBUG,"filter: %s\n",*filter);
   if(port!=NULL){
      aux_port = strtol(port, (char **)NULL, 10);
      switch(errno){
	 case ERANGE:
	    log(ERROR,"Port out of range (%s)\n",port);
	    aux_port=LISTEN_PORT;
	    break;
	 case EINVAL:
	    log(ERROR,"Invalid value for port (%s)\n",port);
	    aux_port=LISTEN_PORT;
	    break;
	 default:
	    break;
      }
   }else{
      aux_port = LISTEN_PORT;
   }
   if (aux_port < 1 || aux_port > 65535) {
      log(ERROR,"Incorrect port number\n");
      exit (1);
   } else {
      *portno = aux_port;
      return;
   }
}

int main(int argc, char **argv) 
{

   int j,k,optval;
   char *filter;
   struct sockaddr_in serv_addr;
   int status;

   lonePacket=alloc_packet_mem(MESSAGE_SIZE);
   devices=(char *)malloc(1500);
   xmlMessage=(char *)malloc(XML_MESSAGE_SIZE);

   memset(serverIp,0,256);
   memset(device,0,DEVICELEN);
   memset(cmdlist,0,sizeof(cmdlist));
   memset(userlist,0,sizeof(userlist));
   pcap_handle=NULL;
   passwordFile=NULL;

   strncpy(device,"any",DEVICELEN-1);
   strcpy(serverIp,"0.0.0.0");//by default, listen in all the interfaces
   parseArgs(argc,argv,&portno,serverIp,&filter,device,DEVICELEN-1);//now, parse the command line arguments
   if(!getDevices(devices,1500))
      log(ERROR,"Unable to get devices ! (are you root?)\n");

   memset(pollFds,0,sizeof(pollFds));//setup the pollFds array cleaning it 
   serverSocketFd = socket(AF_INET, SOCK_STREAM, 0);//open the server socket where clients are going to connect.
   if (serverSocketFd < 0) {
      log(ERROR,"Error trying to open server socket\n");
      exit(1);
   }else{
      log(DEBUG,"serversock=%d\n",serverSocketFd);
   }
   memset((char *) &serv_addr,0, sizeof(serv_addr));//clean the server_address struct
   serv_addr.sin_family = AF_INET;
   if(!inet_aton(serverIp, &(serv_addr.sin_addr))){
      printf("Invalid IP address: %s\n",serverIp);
      exit(0);
   }
   serv_addr.sin_port = htons(portno);
   optval=1;
   if (0>setsockopt(serverSocketFd, SOL_SOCKET, SO_REUSEADDR,(void*)&optval, sizeof(optval))) {
      exit(0);
   }
   if (0>bind(serverSocketFd, (struct sockaddr *) &serv_addr, sizeof(serv_addr))) {
      log(ERROR,"Error trying to bind server Socket (%s)",strerror(errno));
      exit(1);
   }
   sessions = (sessionT **)malloc(MAX_SESSIONS*sizeof(sessionT *));
   signal(SIGINT, &interrupt);
   signal(SIGPIPE, &sigPipe);
   if(0>listen(serverSocketFd, 5)){
      log(ERROR,"Unable to listen (%s)\n",strerror(errno));
      exit(0);
   }
   pollFds[1].fd=serverSocketFd;
   pollFds[1].events=POLLIN|POLLHUP;
   pollFds[1].revents=0;
   if (passwordFile && !(usersNum=processPasswordsFile(passwordFile,userlist,15))){
      log(WARN,"Password file given but no entries processed!\n");
   }
   generate_random_secret();
   if(secure){
      switch(begin_session(device,filter,&pcap_handle)){
	 case SUCCESS:
	    pollFds[0].fd = pcap_get_selectable_fd(pcap_handle);
	    pollFds[0].events = POLLIN|POLLHUP;//pollin received when there are packets captured, pollhup when error or socket close.
	    pollFds[0].revents = 0;//reset the "received events"
	    if(pollFds[0].fd<0){
	       log(ERROR,"Error, cannot select/poll on %s\n",device);
	    }
	    break;
	 case ERR_LOOKUPNET:
	 case ERR_OPEN:
	 case ERR_FILTER:
	    log(ERROR,"Error opening device for capture\n");
	    pcap_handle=NULL;
	    return 0;
      }
      drop_privileges();
   }
   register_command("device",device_command);
   register_command("filter",filter_command);
   register_command("regexp",regexp_command);
   register_command("bye",bye_command);
   register_command("username",username_command);
   register_command("password",password_command);
   register_command("src_ip",src_ip_command);
   register_command("dst_ip",dst_ip_command);
   register_command("save",save_command);
   register_command("replay",replay_command);
   register_command("list",list_command);

   while (TRUE) {
      if(pcap_handle && sessionCount==0 && !secure){
	 pcap_close(pcap_handle);//esto generara un POLLHUP en pollfds[0] (pcap)
	 pcap_handle=NULL;
	 pollFds[0].events=0;
	 pollFds[0].revents=0;
      }
      status = poll(pollFds+(pcap_handle?0:1),sessionCount+1+(pcap_handle?1:0),-1);
      if (status == -1) {
	 if(errno==EINTR)
	    continue;//no need to rebuild tables
	 if(errno==EBADF){
	    log(ERROR,"Bad file descriptor passed to poll !!\n");
	    exit(0);
	 }
	 log(ERROR,"ERROR: poll'ing:%s\n",strerror(errno));
	 status=0;
	 continue;
      } else if (status == 0) { 
	 log(WARN,"WARN: status=0?\n");
	 continue;
      } else {//there are events !
	 for(j=0;j<(sessionCount+2) && status>0;j++){//scan all the sessions for the one that created the event
	    //POLLHUP received when the socket is closed by either side
	    if(pollFds[j].revents & (POLLHUP|POLLIN))
	       status--;
	    if(pollFds[j].revents & POLLIN){
	       pollFds[j].revents &= ~POLLIN;
	       pollFds[j].events = POLLIN|POLLHUP;
	       if(j==0){//the session 0 is the pcap filedescriptor
		  if(0>pcap_dispatch(pcap_handle,-1, my_pcap_handler,NULL)){
		     log(ERROR,"Error on pcap (%s)\n",pcap_geterr(pcap_handle));
		  }
	       }else if(j==1){//if(j==1) new client connecting
		  if (newClient(pollFds[1].fd)) {
		     sessionCount++;
		  }else{
		     log(ERROR,"Error Accepting new client\n");
		  }
	       }else{
		  if(sessions[j-2]->reading_file){
		     reading_file(sessions[j-2]);
		  }else{
		     if(process_command(sessions[j-2])){
			free_session(sessions[j-2]);
			for(k=j;k<sessionCount+1;k++){
			   pollFds[k].fd=pollFds[k+1].fd;
			   pollFds[k].events=pollFds[k+1].events;
			   pollFds[k].revents=pollFds[k+1].revents;
			   sessions[k-2]=sessions[k-1];
			}
			j--;
			sessionCount--;
			continue;
		     }
		  }
	       }
	    }//if(pollFds[j].revents &POLLIN)

	    if(pollFds[j].revents & POLLHUP){ //it could be the pcap handler, finished because no clients left
	       log(DEBUG,"POLLHUP in pollFds[%d] !!\n",j);
	       pollFds[j].revents &= ~POLLHUP;//remove the POLLHUP event only, may remain a POLLIN event
	       pollFds[j].revents &= ~POLLIN; //discard also the POLLIN event, because the FD will be closed
	       pollFds[j].events = POLLIN|POLLHUP;//sanity
	       if(j==0){//if it was the PCAP filedes
		  pcap_handle=NULL;
		  log(WARN,"PCAP disconnected\n");
	       }else if(j>2){//if it was a session FD
		  log(WARN,"CLIENT seems to be disconnected\n");
		  write(sessions[j-2]->controlFd,"</messages>\n",12);//just trying, we will probably receive sigPIPE
		  free_session(sessions[j-2]);
		  sessions[j-2]=NULL;
		  if(pcap_handle && sessionCount==1 && !secure){
		     pcap_close(pcap_handle);
		     pcap_handle=NULL;
		     pollFds[0].events=0;
		  }
	       }
	    }//if pllFds & POLLHUP
	 }//for(all the file descriptors)
      }//else if(status...)
   }//while     
} 


void my_pcap_handler(u_char *user, const struct pcap_pkthdr *header,const u_char *bytes)
{
   int writen,len,i;
   unsigned char packetMatchedFlag;

   packetMatchedFlag=FALSE;
   if(sessionCount==0)
      return;
   lonePacket->errors.s[0]=0;
   lonePacket->type=0;
   if(!capture_packet(bytes,header,lonePacket)){
      return;//if the paket is not sip, discard
   }
   packetMatchedFlag=FALSE; 
   for(i=0;i<sessionCount;i++){
      if(packetMatches(sessions[i],lonePacket)){
	 if(!packetMatchedFlag){
	    if(0>=(len=generateXml(lonePacket,xmlMessage,XML_MESSAGE_SIZE))){
	       log(ERROR,"Unable to generate XML message\n");
	       return;
	    }
	    packetMatchedFlag=TRUE;
	 }
	 log(DEBUG,"writing to controlFd\n");
	 writen = write(sessions[i]->controlFd,xmlMessage,len);
	 //if the controlFd is closed we will receive a SIGPIPE
	 if(writen<0 && errno==EPIPE){
	    log(ERROR,"ERROR writing to controlFd: peer end closed connection\n");
	    free_session(sessions[i]);
	    sessions[i]=NULL;
	    if(pcap_handle && sessionCount==1 && !secure){
	       pcap_close(pcap_handle);
	       pcap_handle=NULL;
	       pollFds[0].events=0;
	    }
	 }
      }
   }
}

void free_session(sessionT *session)
{
   close(session->controlFd);
   if(session->dev.s)
      free(session->dev.s);
   if(session->protocol_version.s)
      free(session->protocol_version.s);
   if(session->filter.s)
      free(session->filter.s);
   if(session->regexp.s)
      free(session->regexp.s);
#if USE_PCRE
   if(session->pattern)
      pcre_free(session->pattern);
   if(session->pattern_extra)
      pcre_free(session->pattern_extra);
#else
   if(session->pattern){
      if(session->pattern->fastmap)
	 free(session->pattern->fastmap);
      free(session->pattern);
   }
#endif
   if(session->clientAddr)
      free(session->clientAddr);
   if(session->username.s)
      free(session->username.s);
   if(session->nonce.s)
      free(session->nonce.s);
   free(session);
}

/**
 * Commands must be < than BUF_LEN
 * If a file command is being read, then the FILE flag must be set to ON,
 * and read_sofar must be set to bytes read so far, and total_length
 * must be set to the total length of the file.
 *
 * Returns 1 if a session has been deleted and a rebuild_tables needs to be performed
 * or 0 if no session has been deleted.
 */
int process_command(sessionT *session) 
{
   int cseq,local_max,n,retval,idx,cmdlen,readbytes;
   char *command,*cmdarg;
   char *car,*ptr,*buffer,*next;

   retval=0;
   buffer=session->buffer;
   idx=session->bufidx;
   next=NULL;
   if(BUF_LEN-idx==0){
      log(ERROR,"Client %.*s has filled the buffer without sending a valid command. Removing it\n",\
	    session->username.len,session->username.s);
      return 1;
   }
   if (0>(n=read(session->controlFd,buffer+idx,BUF_LEN-idx))) {
      log(ERROR,"Error reading from socket (%s) \n",strerror(errno));
      return 1;
   }else if(n==0) {
      log(ERROR,"Socket closed (closing session)\n");
      if(pcap_handle && sessionCount==1 && !secure ){
	 pcap_close(pcap_handle);//esto generara un POLLHUP en pollfds[0] (pcap)
	 pcap_handle=NULL;
	 pollFds[0].events=0;
      }
      write(session->controlFd,"</messages>\n",12);
      return 1;
   }
   log(DEBUG,"received: [%.*s]\n",n+idx,buffer);
   ptr=buffer;
   idx+=n;
   while(ptr<buffer+idx){/* command points to where a new command starts
			  * when we get out of the loop, the remaining data
			  * will be moved to the beginning of the buffer*/
      command=ptr;
      local_max=MAX_COMMAND;
      /*read command name*/
      while(*ptr && *ptr!='\n' && local_max-- && ptr<buffer+idx)
	 ptr++;
      if(ptr==buffer+idx){
	 break;
      }else if(!local_max){
	 log(ERROR,"too long command!\n");
	 goto error;
      }else{
	 *ptr=0;
	 ptr++;
	 car=ptr;
      }
      local_max=MAX_LEN_S;
      /*read command sequence*/
      while(*ptr && *ptr!='\n' && local_max-- && ptr<buffer+idx)
	 ptr++;
      if(ptr==buffer+idx){
	 break;
      }else if(!local_max){
	 log(ERROR,"too long cseq!\n");
	 goto error;
      }else{
	 *ptr=0;
	 ptr++;
	 cseq=atoi(car);
	 car=ptr;
      }
      /**read arg length*/
      local_max=MAX_LEN_S;
      while(*ptr && *ptr!='\n' && local_max-- && ptr<buffer+idx)
	 ptr++;
      if(ptr==buffer+idx){
	 break;
      }else if(!local_max){
	 log(ERROR,"too long length!\n");
	 goto error;
      }else{/*length found*/
	 *ptr=0;
	 cmdlen=atoi(car);
	 ptr++;
	 /*if arg.size < (what remains inside the buffer)*/
	 if(cmdlen<=((buffer+idx)-ptr)){
	    cmdarg=malloc(cmdlen+1);
	    memcpy(cmdarg,ptr,cmdlen);
	    cmdarg[cmdlen]=0;
	    if(ptr+cmdlen<=buffer+idx)
	       next=ptr+cmdlen;
	    switch(execute_command(session,command,cmdarg,cmdlen,cseq)){
	       case CS_SUCCESS:
		  break;
	       case CS_ERROR:
		  log(WARN,"Error executing command %s with args:[%.*s]\n",command,cmdlen,ptr);
		  break;
	       case CS_REMOVE:
		  write(session->controlFd,"</messages>\n",12);
		  retval=1;
		  break;
	       case CS_UNKNOWN:
		  log(WARN,"Unkown command: %s args=[%s]\n",command,cmdarg);
		  writeMsg(session->controlFd,"<response status=\"unknown\" reason=\"");
		  for(n=0;n<cmdnr;n++){
		     writeMsg(session->controlFd,cmdlist[n].cmdname);
		     write(session->controlFd,", ",2);
		  }
		  writeMsg(session->controlFd,"\"></response>\n");
		  break;
	       case CS_FILE:
		  ptr+=cmdlen;
		  write(session->file.fd,ptr,(buffer+idx)-ptr);
		  session->bufidx=0;
		  session->reading_file=1;
		  return 0;
		  break;
	       default:
		  log(ERROR,"unknown response type !!!\n");
		  break;
	    }
	    ptr+=cmdlen;
	 }else{
	    break;
	 }
      }
   }
   if(next){/*at least, 1 command has been processed*/
      if(next<buffer+idx){
	 session->bufidx=(buffer+idx)-next;
	 memmove(buffer,next,session->bufidx);
      }else{
	 session->bufidx=0;
      }
   }else{
      session->bufidx=idx;
   }
   return retval;
error:
   log(ERROR,"ERROR in process_command\n");
   return 1;
}

int reading_file(sessionT* session)
{
   int n,i,towrite,toread,fileremaining,written;

   towrite=0;
   fileremaining=session->file.total_len - session->file.read_sofar;
   
   if(0>(n=read(session->controlFd,session->buffer,BUF_LEN))) {
      log(ERROR,"Error reading from socket (%s) \n",strerror(errno));
      return 1;
   }else if(n==0) {
      log(ERROR,"Socket closed (closing session)\n");
      if(pcap_handle && sessionCount==1 && !secure ){
	 pcap_close(pcap_handle);//esto generara un POLLHUP en pollfds[0] (pcap)
	 pcap_handle=NULL;
	 pollFds[0].events=0;
      }
      write(session->controlFd,"</messages>\n",12);
      return 1;
   }
   if(fileremaining < n){
      towrite=fileremaining;
   }else
      towrite=n;
   written=0;
   while(towrite){
      if(0>(i=write(session->file.fd,session->buffer+written,towrite))){
	 return 1;
      }else{
	 towrite-=i;
	 written+=i;
      }
   };
   if(fileremaining <= n){/*we're done*/
      close(session->file.fd);
      session->reading_file=0;
      memmove(session->buffer,session->buffer+fileremaining,n-fileremaining);
      session->bufidx=n-fileremaining;
   }else{
      session->file.read_sofar+=n;
   }
   return 0;
}

/**
 * Returns 1 if a session has been deleted and a rebuild_tables needs to be performed
 * or 0 if no session has been deleted.
 */


int newClient(int sockfd)
{
   int ctrlSockFd,flag;
   socklen_t clilen;
   struct sockaddr_in cli_addr;

   clilen=sizeof(struct sockaddr_in);

   ctrlSockFd = accept(sockfd, (struct sockaddr *) &cli_addr, &clilen);
   if (ctrlSockFd < 0) {
      printf("Error accepting connection in serverSocket fd=%d (%s)\n",sockfd,strerror(errno));
      return 0;
   }

   if(sessionCount == MAX_SESSIONS){
      log(WARN,"Maximum number of clients reached. Rejecting client %s:%d\n", \
	    inet_ntoa(cli_addr.sin_addr),ntohs(cli_addr.sin_port));
      close(ctrlSockFd);
      return 0;
   }

   log(DEBUG,"New client from %s:%d\n",inet_ntoa(cli_addr.sin_addr),ntohs(cli_addr.sin_port));

   flag = 1;
#ifdef SET_TCP_NODELAY
   if (0>setsockopt(ctrlSockFd, IPPROTO_TCP, TCP_NODELAY, (char *)&flag, sizeof(flag))) {
      log(WARN,"Unable to set TCP_NODELAY !\n");
   }
#endif

   if (!(sessions[sessionCount]=(sessionT *)malloc(sizeof(sessionT)))) {
      log(ERROR,"Out of memory for sessionT(%d) !!\n",sizeof(sessionT));
      close(ctrlSockFd);
      return 0;
   }
   memset(sessions[sessionCount],0,sizeof(sessionT));
   if (!(sessions[sessionCount]->clientAddr =(struct sockaddr *) malloc(sizeof(struct sockaddr)))) {
      log(ERROR,"Out of memory for sessionT->clientAddr (%d) !!\n",sizeof(struct sockaddr));
      free(sessions[sessionCount]);
      close(ctrlSockFd);
      return 0;
   }
   if (!(sessions[sessionCount]->buffer=(char *)malloc(BUF_LEN))) {
      log(ERROR,"Out of memory for sessionT->buffer (%d) !!\n",BUF_LEN);
      free(sessions[sessionCount]->clientAddr);
      free(sessions[sessionCount]);
      close(ctrlSockFd);
      return 0;
   }
   writeMsg(ctrlSockFd,"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n<messages>\n");
   pollFds[sessionCount+2].fd = ctrlSockFd;
   pollFds[sessionCount+2].events = POLLIN|POLLHUP;
   pollFds[sessionCount+2].revents = 0;
   sessions[sessionCount]->controlFd=ctrlSockFd;
   sessions[sessionCount]->clientID=clientId++;
   memcpy(sessions[sessionCount]->clientAddr,&cli_addr,clilen);
   sessions[sessionCount]->expires=time(0)+20;
   return 1;
}

void disconnectClients()
{
   int i=0;
   for(i=0;i<sessionCount;i++){
      close(sessions[i]->controlFd);
   }
}

char *get_filter_from_argv(char **argv) 
{
   char **arg = argv, *theirs, *mine;
   char *from, *to;
   uint32_t len = 0;

   if (!*arg)
      return NULL;

   while (*arg)
      len += (uint32_t)strlen(*arg++) + 1; // *arg++ = increment arg, then return the value of (arg+1)pointer
   //previous sentence sums up all the argument's length: 
   //more or less: strlen(argv[1])+strlen(argv[2])+strlen(argv[3])...

   if (!(theirs = (char*)malloc(len + 1)) ||
	 !(mine = (char*)malloc(len + sizeof(BPF_MAIN_FILTER))))
      return NULL;

   memset(theirs, 0, len + 1);
   memset(mine, 0, len + sizeof(BPF_MAIN_FILTER));

   arg = argv;
   to = theirs;

   while ((from = *arg++)) { //for(int i=0;i<argc;i++) from+=argv[i]; more or less
      while ((*to++ = *from++));
      *(to-1) = ' ';
   }

   sprintf(mine, BPF_MAIN_FILTER, theirs);
   free(theirs);
   return mine;
}

/**Warn, this function is NOT reentrant, and returns a pointer to a static structure..
 * This function does NOT support IPv6
 */
char* printAddrIp4(pcap_addr_t *a)
{
   static char address[100];
   char *family;
   void *the_in_addr=0;
   pcap_addr_t *myAddr;

   if(!a)
      return 0;
   else
      myAddr=a;

   *address=0;
   for(;myAddr;myAddr=myAddr->next) {
      if(myAddr->addr==NULL)
	 continue;
      switch(myAddr->addr->sa_family){
	 case AF_INET:
	 case AF_INET6:
	    if (myAddr->addr->sa_family==AF_INET){
	       the_in_addr=(void *)&((struct sockaddr_in *)myAddr->addr)->sin_addr;
	       family="IPv4";
	    }else if(myAddr->addr->sa_family==AF_INET6){
	       the_in_addr=(void *)&((struct sockaddr_in6 *)myAddr->addr)->sin6_addr;
	       family="IPv6";
	    }else{
	       log(ERROR,"sa_family null !\n");
	       return 0;
	    }
	    /* second argument to inet_ntop must be the address in NBO, if this OS supports
	     * BSD-style "stuct sockaddr" there's no problem(hope..).
	     * There's some mess with IPv6 and libpcap. See
	     * http://www.ipv6style.jp/en/apps/20030829/index.shtml
	     * http://www.tcpdump.org/lists/workers/2002/07/msg00063.html
	     */
	    if (!inet_ntop(myAddr->addr->sa_family,the_in_addr,address,100)) {
	       log(ERROR,"ERROR translating %s address to ASCII\n",family);
	       goto error;
	    }else{
	       log(DEBUG,"Address is %s\n",address);
	       goto done;
	    }
	    break;
	 case AF_UNSPEC:
	    family="AF_UNSPEC";
	    goto error;
	 case AF_LOCAL:
	    family="AF_LOCAL";
	    goto error;
	 case AF_IPX:
	    family="AF_IPX";
	    goto error;
	 case AF_APPLETALK:
	    family="AF_APPLETALK";
	    goto error;
	 case AF_SNA:
	    family="AF_SNA";
	    goto error;
	 case AF_MAX:
	    family="AF_MAX";
	    goto error;
	 default:
	    family="UNKNOWN";
	    continue;
      }/*switch*/
      break;
   }/*for*/
done:
   return address;
error:
   snprintf(address,100,"(%s family address)",family);
   return address;
}

void drop_privileges()
{
   struct passwd *pw;
   uid_t newuid;
   gid_t newgid;

   if (getuid() || geteuid())
      return;

   pw = getpwnam("nobody");
   if (!pw) {
      perror("Privilege drop failed ");
      exit(-1);
   }

   newgid = pw->pw_gid;
   newuid = pw->pw_uid;

   if (getgroups(0, NULL) > 0)
      if (setgroups(1, &newgid) == -1) {
	 perror("Privilege drop failed");
	 exit(-1);
      }

   if (((getgid()  != newgid) && (setgid(newgid)  == -1)) ||
	 ((getegid() != newgid) && (setegid(newgid) == -1)) ||
	 ((getuid()  != newuid) && (setuid(newuid)  == -1)) ||
	 ((geteuid() != newuid) && (seteuid(newuid) == -1))) {

      perror("Privilege drop failed ");
      exit(-1);
   }
}

int getDevices(char *buf,int len)
{
   char errbuf[PCAP_ERRBUF_SIZE];
   char *ipaddr,*devname;
   pcap_if_t *devices,*devicesOrig;
   pcap_addr_t *addrX;
   unsigned short int i;
   if(pcap_findalldevs(&devices,errbuf)==0){
      devicesOrig=devices;
      i=sprintf(buf,"<devices>\n");
      for(;devices;devices=devices->next){
	 addrX=devices->addresses;
	 devname=devices->name?devices->name:"(unknown)";
	 if (!(ipaddr=printAddrIp4(addrX))) {
	    log(ERROR,"Unable to retrieve IP address for interface %s\n",devname);
	    ipaddr="X.X.X.X";
	 }else{
	    log(WARN,"Interface %s with IP address: %s\n",devname,ipaddr);
	 }
	 if(secure){
	    if (!strcmp(devname,device)) {
	       i+=sprintf(buf+i,"<device name=\"%s\" ip=\"%s\" />\n",devname,ipaddr);
	    }
	 }else{
	    i+=sprintf(buf+i,"<device name=\"%s\" ip=\"%s\" />\n",devname,ipaddr);
	 }
      }
      if(file)
	 i+=sprintf(buf+i,"<device name=\"file:%s\"/>\n",file);
      sprintf(buf+i,"</devices>\n");
      pcap_freealldevs(devicesOrig);
   }else{
      return 0;
   }
   log(DEBUG,"devices:%s\n",buf);
   return 1;
}

int processPasswordsFile(char *filename,userT *list,int listLen)
{
   int read,i,j;
   struct stat filestat;
   FILE *passFile;
   char *buffer,*name,*pass,*role,*regexp;

   if(filename!=NULL){
      if (0>stat(filename,&filestat)){
	 log(ERROR,"Unable to stat password file (%s)\n",strerror(errno));
	 return 0;
      }else if (filestat.st_size > MAX_FILE_SIZE) {
	 log(ERROR,"Password file too big (> MAX_FILE_SIZE bytes)\n");
	 return 0;
      }
      if (!(buffer=malloc(filestat.st_size))) {
	 log(ERROR,"Unable to malloc %d bytes\n",(int)filestat.st_size);
	 return 0;
      }
      if (!(passFile=fopen(filename,"r"))) {
	 log(ERROR,"Error opening passwords file %s (%s)\n",filename,strerror(errno));
	 goto error;
      }
      read=fread(buffer,1,filestat.st_size,passFile);
      if(read<0 || read<filestat.st_size){
	 log(ERROR,"Unable to read from passwords file (%s)\n",filename);
	 goto error;
      }
      i=j=0;
      name=buffer;
      pass=role=regexp=0;
      while(j<read && i<listLen){
	 for(;j<read && buffer[j]!=':' && buffer[j]!='\n';j++);
	 if(j==read || 
	       (buffer[j]=='\n' && !(regexp && pass && role)) || 
	       (buffer[j]==':' && (regexp && pass && role))){
	    log(ERROR,"Malformed password file (needs to be (username:password:role:regexp\\n)*)\n");
	    goto error;
	 }
	 if(buffer[j]=='\n'){
	    list[i].username=name;
	    list[i].password=pass;
	    list[i].role=role;
	    list[i].regexp=regexp;
	    buffer[j]=0;
	    pass=role=regexp=0;
	    name=buffer+j+1;
	    if (!calcRegex(list+i)) {
	       log(ERROR,"Error calculating regexp\n");
	       goto error;
	    }
	    i++;
	 }else if(buffer[j]==':'){
	    buffer[j]=0;
	    if (!pass) {
	       pass=buffer+j+1;
	    }else if (!role) {
	       role=buffer+j+1;
	    }else if (!regexp){
	       regexp=buffer+j+1;
	    }
	 }
	 j++;
      }
      log(DEBUG,"users/passwords/roles/regexp:\n");
      fclose(passFile);
      if(debug_level>=DEBUG){
	 for(j=0;j<i;j++)
	    log(DEBUG,"%s:%s:%s:%s\n",list[j].username,list[j].password,list[j].role,list[j].regexp);
      }
   }
   return i;
error:
   fclose(passFile);
   free(buffer);
   return 0;
}

static inline int calcRegex(userT *usr)
{
   const char *re_err;
#if USE_PCRE
   int32_t err_offset;
   uint32_t pcre_options = PCRE_UNGREEDY;
#else
   re_syntax_options = RE_CHAR_CLASSES | RE_NO_BK_PARENS | RE_NO_BK_VBAR | RE_CONTEXT_INDEP_ANCHORS | RE_CONTEXT_INDEP_OPS;
#endif
#if USE_PCRE
   re_err=NULL;
   log(DEBUG,"Compiling pcre[%s]\n",usr->regexp);
   if (!(usr->pattern=pcre_compile(usr->regexp, pcre_options, (const char **)&re_err, &err_offset, 0))) {
      log(ERROR, "Failed Compilation: %s\n", re_err);
      goto error;
   }
   usr->pattern_extra = pcre_study(usr->pattern, 0, (const char **)&re_err);
   if (re_err) {
      log(ERROR,"PCRE Study failed:%s\n",re_err);
      usr->pattern_extra=NULL;
   }
#else
   if(!(usr->pattern =malloc(sizeof(struct re_pattern_buffer)))){
      log(ERROR,"Out of memory!!\n");
      goto error;
   }
   memset(usr->pattern,0,sizeof(struct re_pattern_buffer));
   if(!(usr->pattern->fastmap =malloc(MAX_PATTERN_LEN))){
      log(ERROR,"Out of memory!!\n");
      goto error;
   }
   memset(usr->pattern->fastmap,0,MAX_PATTERN_LEN);
   log(DEBUG,"Compiling gnu-regex[%s]\n",usr->regexp);
   re_err = re_compile_pattern(usr->regexp,strlen(usr->regexp)+1,usr->pattern);
   if (re_err) {
      log(ERROR, "regex compile: %s\n", re_err);
      goto error;
   }
   if (re_compile_fastmap(usr->pattern)) {
      log(ERROR,"fastmap compile failed");
      goto error;
   }
#endif
   return 1;
error:
#if USE_PCRE
   if(usr->pattern){
      pcre_free(usr->pattern);
      usr->pattern=NULL;
   }
   if(usr->pattern_extra){
      pcre_free(usr->pattern_extra);
      usr->pattern_extra=NULL;
   }
#else
   if(usr->pattern){
      if(usr->pattern->fastmap){
	 free(usr->pattern->fastmap);
	 usr->pattern->fastmap=NULL;
      }
      free(usr->pattern);
      usr->pattern=NULL;
   }
#endif
   return 0;
}

/**
 *
 */
int register_command(char *cmdname,command_cb callback)
{
   int i;
   log(DEBUG,"Registering command [%s]\n",cmdname);
   if(cmdnr==MAX_COMMAND)
      return 0;
   for(i=0;i<cmdnr;i++){
      if (!strcmp(cmdlist[i].cmdname,cmdname)) {
	 return 0;
      }
   }
   cmdlist[cmdnr].cmdname=cmdname;
   cmdlist[cmdnr].callback=callback;
   cmdnr++;
   return 1;
}

/*
 * args must also be null-terminated
 * arglen must be the length of the arg (without the nulltermination)
 */
enum cmdStat execute_command(sessionT *session, char *cmdname,char *args,int arglen,int cseq)
{
   int i;
   str tmp;
   tmp.s=args;
   tmp.len=arglen;
   for(i=0;i<cmdnr;i++){
      if (!strcmp(cmdlist[i].cmdname,cmdname)) {
	 return cmdlist[i].callback(session,tmp,cseq);
      }
   }
   return CS_UNKNOWN;
}
