/*
 * Copyright (C) 2004-2007 VozTelecom Sistemas
 *
 * This file is part of SipSpy, a SIP monitoring tool.
 *
 * SipSpy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * SipSpy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef SPYAGENT_H
#define SPYAGENT_H

#include <sys/poll.h>
#include <pcap.h>
#include <netinet/in.h>/*in_addr*/

#if USE_PCRE
#include "pcre.h"
#endif

#define ETHHDR_SIZE 14
#define TOKENRING_SIZE 22
#define PPPHDR_SIZE 4
#define SLIPHDR_SIZE 16
#define RAWHDR_SIZE 0
#define LOOPHDR_SIZE 4
#define FDDIHDR_SIZE 21
#define ISDNHDR_SIZE 16
#define IEEE80211HDR_SIZE 32

#define LISTEN_PORT	23332
#define MAX_SESSIONS  20
#define IP_PORT_MAX_LEN	60 //just in case someone uses IPv6 over there
#define BRANCH_ID_MAX_LEN 120
#define CALL_ID_MAX_LEN 120
#define FIRST_LINE_MAX_LEN 150
#define MESSAGE_MAX_LEN 1500
#define STATE_WAIT	  1
#define STATE_CAPTURE	2
#define EVER ;;
#define MAX_USERNAME 15
#define DEVICELEN 15

#define MESSAGE_SIZE  1550
#define XML_MESSAGE_SIZE 2500

#define MAX_PATTERN_LEN 256

#define TRUE  1
#define FALSE 0


#define MAX_FILE_SIZE 5000

#define BPF_FILTER_IP       "(ip)"
#define BPF_FILTER_OTHER    " and ( %s)"
#define BPF_MAIN_FILTER     BPF_FILTER_IP BPF_FILTER_OTHER

#define ADMIN_ROLE 0x01
#define ADMIN_ROLE_S "admin"
#define USER_ROLE 0x02
#define USER_ROLE_S "user"

#define log(severity, format, args...) \
   do{ \
      if(severity>=debug_level) \
      printf(format, ## args); \
   }while(0)

#define writeMsg(fd, msg) \
   do{\
      write(fd, msg, strlen(msg));\
   }while(0)

#define DEBUG 0
#define INFO 1
#define WARN 2
#define ERROR 3
#define FATAL 4

#define SIP_REQUEST 1
#define SIP_RESPONSE 2

#define MAX_CHUNK 1500

typedef struct {
   char *username;
   char *password;
   char *role;
   char *regexp;
#if USE_PCRE
   pcre *pattern;
   pcre_extra *pattern_extra;
#else
   struct re_pattern_buffer *pattern;
#endif
}userT;

typedef struct {
   char *s;
   int len;
}str;

struct file{
   int fd;
   unsigned int total_len;
   unsigned int read_sofar;
};

typedef struct {
   int controlFd;
   int clientID;
   str protocol_version;
   str owner;
   str dev;
   str filter;
   str regexp;
   str username;
   str password;
   str nonce;
   struct file file;
   struct in_addr src_ip;
   struct in_addr dst_ip;
   char role;
   char reading_file;
   char authenticated;
   time_t expires;
   char *buffer;
   int bufidx;
#if USE_PCRE
   pcre *pattern;
   pcre_extra *pattern_extra;

   pcre *regex_pattern;
   pcre_extra *regex_pattern_extra;
#else
   struct re_pattern_buffer *pattern;
   struct re_pattern_buffer *regex_pattern;
#endif
   struct sockaddr *clientAddr;
}sessionT;

typedef struct {
   char from[INET6_ADDRSTRLEN + 1];
   char to[INET6_ADDRSTRLEN + 1];
   unsigned short int fromPort;
   unsigned short int toPort;
   long seconds;	
   long microseconds;	
   char isSender;
   char type;
   str transactionId;
   str callId;
   str firstLine;
   str firstViaHost;
   str firstViaPort;
   str errors;
   str message;
}packetT;

extern int debug_level;
extern int secure;
extern int portno;
extern char *devices;
extern int usersNum;
extern userT userlist[15];
extern int sessionCount;
extern pcap_t *pcap_handle;
extern struct pollfd pollFds[MAX_SESSIONS+2];

typedef enum cmdStat (command_cb) (sessionT *s,str arg,int cseq);

void my_pcap_handler(u_char *user, const struct pcap_pkthdr *header,const u_char *bytes);
char* printAddrIp4(pcap_addr_t *myAddr);
int newClient(int fd);
void disconnectClients();
char *get_filter_from_argv(char **argv);
int process_command (sessionT *session);
int reading_file(sessionT* session);
void free_session(sessionT *session);
void drop_privileges();
int getDevices(char* devices,int len);
int processPasswordsFile(char *filename,userT *list,int listLen);
enum cmdStat execute_command(sessionT *session,char* cmdname,char* args,int arglen,int cseq);
int register_command(char* cmdname,command_cb *callback);
#endif
